import sys
from twisted.internet import reactor
from BitTorrent.defer import Deferred

class ThreadProxy(object):
    __slots__ = ('obj', 'queue_task')
    def __init__(self, obj, queue_task):
        self.obj = obj
        self.queue_task = queue_task

    def __gen_call_wrapper__(self, f):
        def outer(*args, **kwargs):
            df = Deferred()
            def inner(*args, **kwargs):
                try:
                    v = f(*args, **kwargs)
                except:
                    # hm, the exc_info holds a reference to the deferred, I think
                    self.queue_task(df.errback, sys.exc_info())
                else:
                    if isinstance(v, Deferred):
                        # v is owned by the proxied thread, so add the callback
                        # now, but the task itself should queue for the caller
                        # thread
                        v.addCallback(lambda r : self.queue_task(df.callback, r))
                    else:
                        self.queue_task(df.callback, v)
            reactor.callFromThread(inner, *args, **kwargs)
            return df
        return outer

    def __getattr__(self, attr):
        a = getattr(self.obj, attr)
        if callable(a):
            return self.__gen_call_wrapper__(a)
        return a

    def call_with_obj(self, _f, *a, **k):
        w = self.__gen_call_wrapper__(_f)
        return w(self.obj, *a, **k)





# The contents of this file are subject to the BitTorrent Open Source License
# Version 1.1 (the License).  You may not copy or use this file, in either
# source code or executable form, except in compliance with the License.  You
# may obtain a copy of the License at http://www.bittorrent.com/license/.
#
# Software distributed under the License is distributed on an AS IS basis,
# WITHOUT WARRANTY OF ANY KIND, either express or implied.  See the License
# for the specific language governing rights and limitations under the
# License.

# written by Matt Chisholm

import os
import pickle
import logging
from BitTorrent.translation import _

from BitTorrent import zurllib
from BitTorrent import GetTorrent
from BitTorrent import app_name, version, BTFailure

from BitTorrent.hash import sha
from BitTorrent.ConvertedMetainfo import ConvertedMetainfo
from BitTorrent.bencode import bdecode, bencode
from BitTorrent.platform import osx, get_temp_dir, doc_root, os_version
from BitTorrent.yielddefer import launch_coroutine
from BitTorrent.defer import Deferred, ThreadedDeferred
from BitTorrent.platform import osx, get_temp_dir, doc_root
from BitTorrent.yielddefer import launch_coroutine, _wrap_task
from BitTorrent.MultiTorrent import TorrentAlreadyRunning, TorrentAlreadyInQueue, UnknownInfohash
from BitTorrent.obsoletepythonsupport import set

from TorrentButler import TorrentButler
from NewVersion import Version


version_host = 'http://version.bittorrent.com/'
download_url = 'http://bittorrent.com/download.html'

DEBUG = False

class AutoUpdateButler(TorrentButler):

    def __init__(self, multitorrent, rawserver,
                 test_new_version=None, test_current_version=None):
        TorrentButler.__init__(self, multitorrent)

        self.runs = 0

        self.rawserver = rawserver
        self.estate = set()

        self.log_root = "core.AutoUpdateButler"
        self.logger = logging.getLogger(self.log_root)

        self.installable_version = None
        self.available_version   = None

        self.current_version = Version.from_str(version)

        self.debug_mode = DEBUG

        self.delay = 60*60*24
        if self.debug_mode:
            self.delay = 10

        if test_new_version:
            test_new_version = Version.from_str(test_new_version)
            self.debug_mode = True
            self.debug('__init__() turning debug on')
            def _hack_get_available(url):
                self.debug('_hack_get_available() run#%d: returning %s' % (self.runs, str(test_new_version)))
                return test_new_version
            self._get_available = _hack_get_available
        if test_current_version:
            self.debug_mode = True
            self.current_version = Version.from_str(test_current_version)

        self.version_site = version_host
        # The version URL format is:
        # http:// VERSION_SITE / OS_NAME / (LEGACY /) BETA or STABLE

        # LEGACY means that the user is on a version of an OS that has
        # been deemed "legacy", and as such the latest client version
        # for their OS version may be different than the latest client
        # version for the OS in general.  For example, if we are going
        # to roll a version that requires WinXP/2K or greater, or a
        # version that requires OSX 10.5 or greater, we may maintain
        # an older version for Win98 or OSX 10.4 in OS_NAME/legacy/.

        if os.name == 'nt':
            self.version_site += 'win32/'
            if os_version not in ('XP', '2000', '2003'):
                self.version_site += 'legacy/'
        elif osx:
            self.version_site += 'osx/'
        elif self.debug_mode:
            self.version_site += 'win32/'

        self.installer_dir = self._calc_installer_dir()


    def get_auto_update_status(self):
        r = None, None
        if not self._can_install():
            # Auto-update doesn't work here, so just notify the user
            # of the new available version.
            r = self.available_version, None, self.delay
            self.available_version = None
        elif self.installable_version is not None:
            # Auto-update is done, notify the user of the version
            # ready to install.
            r = self.available_version, self.installable_version, self.delay
            self.available_version = None
        else:
            # Auto-update is in progress, don't tell the user
            # anything, and don't reset anything.
            r = None, None, None
        return r


    def butle(self):
        for i in list(self.estate):
            try:
                t = self.multitorrent.get_torrent(i)
                if t.state == 'initialized':
                    self.multitorrent.start_torrent(t.infohash)
                torrent, status = self.multitorrent.torrent_status(i)
                if torrent.completed:
                    self.finished(t)
            except UnknownInfohash:
                self.debug('butle() removing' + i.short())
                self.estate.remove(i)
                self.installable_version = None
                self.available_version = None


    def butles(self, torrent):
        return torrent.infohash in self.estate and torrent.is_initialized()


    def started(self, torrent):
        """Only run the most recently added torrent"""
        if self.butles(torrent):
            removable = self.estate - set([torrent.infohash])
            for i in removable:
                self.estate.discard(i)
                self.multitorrent.remove_torrent(i)
                # BUG: remove unfinished download of obsolete auto-update from disk


    def finished(self, torrent):
        """Launch the auto-updater"""
        self.debug('finished() called for '+torrent.infohash.short())
        if self.butles(torrent):
            self.debug('finished() setting installable version to '+ torrent.infohash.short())
            self.installable_version = torrent.infohash


    # Auto-update specific methods
    def debug(self, message):
        if self.debug_mode:
            self.logger.warning(message)


    def _can_install(self):
        """Return True if this OS supports auto-updates."""
        if self.debug_mode:
            return True
        if self.installer_dir is None:
            return False

        if os.name == 'nt':
            return True
        elif osx:
            return True
        else:
            return False


    def _calc_installer_name(self, available_version):
        """Figure out the name of the installer for this OS."""
        if os.name == 'nt' or self.debug_mode:
            ext = 'exe'
        elif osx:
            ext = 'dmg'
        elif os.name == 'posix':
            ext = 'tar.gz'
        else:
            return

        parts = [app_name, str(available_version)]
        if available_version.is_beta():
            parts.append('Beta')
        name = '-'.join(parts)
        name += '.' + ext
        return name


    def _calc_installer_dir(self):
        """Find a place to store the installer while it's being downloaded."""
        temp_dir = get_temp_dir()
        return temp_dir


    def _get_available(self, url):
        """Get the available version from the version site.  The
        command line option --new_version X.Y.Z overrides this method
        and returns 'X.Y.Z' instead."""
        self.debug('_get_available() run#%d: hitting url %s' % (self.runs, url))
        try:
            u = zurllib.urlopen(url)
            s = u.read()
            s = s.strip()
        except:
            raise BTFailure(_("Could not get latest version from %s")%url)
        try:
            assert len(s) == 5
            available_version = Version.from_str(s)
        except:
            raise BTFailure(_("Could not parse new version string from %s")%url)

        return available_version


    def _get_torrent(self, installer_url):
        """Get the .torrent file from the version site."""
        torrentfile = None
        try:
            torrentfile = GetTorrent.get_url(installer_url)
        except GetTorrent.GetTorrentException, e:
            self.debug('_get_torrent() run#%d: failed to download torrent file %s: %s' %
                       (self.runs, installer_url, unicode(e.args[0])))
            pass
        return torrentfile


    def _get_signature(self, installer_url):
        """Get the signature (.sign) file from the version site, and
        unpickle the signature.  The sign file is a signature of the
        .torrent file created with the auto-update tool in
        auto-update/sign_file.py."""
        signature = None
        try:
            signfile = zurllib.urlopen(installer_url + '.sign')
        except:
            self.debug('_get_signature() run#%d: failed to download signfile %s.sign' %
                       (self.runs, installer_url))
            pass
        else:
            try:
                signature = pickle.load(signfile)
            except:
                self.debug('_get_signature() run#%d: failed to unpickle signfile %s' %
                           (self.runs, signfile))
                pass
        return signature


    def _check_signature(self, torrentfile, signature):
        """Check the torrent file's signature using the public key."""
        public_key_file = open(os.path.join(doc_root, 'public.key'), 'rb')
        public_key = pickle.load(public_key_file)
        public_key_file.close()
        h = sha(torrentfile).digest()
        return public_key.verify(h, signature)


    def check_version(self):
        """Launch the actual version check code in a coroutine since
        it needs to make three (or four, in beta) http requests, one
        disk read, and one decryption."""
        df = launch_coroutine(_wrap_task(self.rawserver.external_add_task),
                              self._check_version)
        def errback(e):
            from traceback import format_exc
            self.debug('check_version() run #%d: '%self.runs + format_exc(e))
        df.addErrback(errback)


    def _check_version(self):
        """Actually check for an auto-update:
        1.  Check the version number from the file on the version site.
        2.  Check the stable version number from the file on the version site.
        3.  Notify the user and stop if they are on an OS with no installer.
        4.  Get the torrent file from the version site.
        5.  Get the signature from the version site.
        6.  Check the signature against the torrent file using the public key.
        7a. Start the torrent if it's not in the client.
        7b. Restart the torrent if it's in the client but not running.
        8.  Put the infohash of the torrent into estate so the butler knows
            to butle it.
        9.  AutoUpdateButler.started() ensures that only the most recent
            auto-update torrent is running.
        10. AutoUpdateButler.finished() indicates the new version is available,
            the UI polls for that value later.

        Whether an auto-update was found and started or not, requeue
        the call to check_version() to run a day later.  This means
        that the version check runs at startup, and once a day.
        """
        debug_prefix = '_check_version() run#%d: '%self.runs
        self.debug(debug_prefix + 'starting')

        url = self.version_site + self.current_version.name()

        df = ThreadedDeferred(_wrap_task(self.rawserver.external_add_task),
                              self._get_available, url, daemon=True)
        yield df
        try:
            available_version = df.getResult()
        except BTFailure, e:
            self.debug(debug_prefix + 'failed to load %s' % url)
            self._restart()
            return

        if available_version.is_beta():
            if available_version[1] != self.current_version[1]:
                available_version = self.current_version
        if self.current_version.is_beta():
            stable_url = self.version_site + 'stable'
            df = ThreadedDeferred(_wrap_task(self.rawserver.external_add_task),
                                  self._get_available, stable_url)
            yield df
            try:
                available_stable_version = df.getResult()
            except BTFailure, e:
                self.debug(debug_prefix + 'failed to load %s' % url)
                self._restart()
                return

            if available_stable_version > available_version:
                available_version = available_stable_version
        self.debug(debug_prefix + 'got %s' % str(available_version))


        if available_version <= self.current_version:
            self.debug(debug_prefix + 'not updating old version %s' %
                       str(available_version))
            self._restart()
            return

        if not self._can_install():
            self.debug(debug_prefix + 'cannot install on this os')
            self.available_version = available_version
            self._restart()
            return

        installer_name = self._calc_installer_name(available_version)
        installer_url  = self.version_site + installer_name + '.torrent'
        installer_path = unicode(os.path.join(self.installer_dir, installer_name))

        df = ThreadedDeferred(_wrap_task(self.rawserver.external_add_task),
                              self._get_torrent, installer_url)
        yield df
        torrentfile = df.getResult()

        df = ThreadedDeferred(_wrap_task(self.rawserver.external_add_task),
                              self._get_signature, installer_url)
        yield df
        signature = df.getResult()

        if torrentfile and signature:
            df = ThreadedDeferred(_wrap_task(self.rawserver.external_add_task),
                                  self._check_signature, torrentfile, signature)
            yield df
            checked = df.getResult()
            if checked:
                self.debug(debug_prefix + 'signature verified successfully.')
                b = bdecode(torrentfile)
                metainfo = ConvertedMetainfo(b)
                infohash = metainfo.infohash
                self.available_version = available_version

                self.multitorrent.remove_auto_updates_except(infohash)

                try:
                    df = self.multitorrent.create_torrent(metainfo, installer_path, installer_path, hidden=True, is_auto_update=True)
                    yield df
                    df.getResult()
                except TorrentAlreadyRunning:
                    self.debug(debug_prefix + 'found auto-update torrent already running')
                except TorrentAlreadyInQueue:
                    self.debug(debug_prefix + 'found auto-update torrent queued')
                else:
                    self.debug(debug_prefix + 'starting auto-update download')

                self.debug(debug_prefix + 'adding to estate '+infohash.short())
                self.estate.add(infohash)

            else:
                self.debug(debug_prefix + 'torrent file signature failed to verify.')
                pass


        else:
            self.debug(debug_prefix + 'couldn\'t get both torrentfile %s and signature %s' %
                       (str(type(torrentfile)), str(type(signature))))

        self._restart()


    def _restart(self):
        """Run the auto-update check once a day, or every ten seconds
        in debug mode."""
        self.runs += 1
        self.rawserver.external_add_task(self.delay, self.check_version)



# a very simple (and silly) mechanism for getting the host_ip

import socket
from BitTorrent.platform import bttime
from BitTorrent.obsoletepythonsupport import set

__host_ip = None
__host_ip_cachetime = 0
__host_ips = None
__host_ips_cachetime = 0
CACHE_TIME = 300

def get_host_ip():
    global __host_ip
    global __host_ip_cachetime

    if __host_ip is not None and __host_ip_cachetime + CACHE_TIME > bttime():
        return __host_ip
    
    #try:
    #    ip = socket.gethostbyname(socket.gethostname())
    #except socket.error, e:
    # mac sometimes throws an error, so they can just wait.
    # plus, complicated /etc/hosts will return invalid IPs

    try:
        s = socket.socket(socket.AF_INET, socket.SOCK_STREAM)
        s.connect(("bittorrent.com", 80))
        endpoint = s.getsockname()
        __host_ip = endpoint[0]
    except socket.error, e:
        __host_ip = socket.gethostbyname(socket.gethostname())

    return __host_ip

def get_host_ips():
    global __host_ips
    global __host_ips_cachetime

    if __host_ips is not None and __host_ips_cachetime + CACHE_TIME > bttime():
        return __host_ips
    
    l = set()

    host_ip = get_host_ip()
    if host_ip is not None:
        l.add(host_ip)

    try:
        hostname = socket.gethostname()
        hostname, aliaslist, ipaddrlist = socket.gethostbyname_ex(hostname)
        l.update(ipaddrlist)
    except socket.error, e:
        print "ARG", e        

    __host_ips = l
    __host_ips_cachetime = bttime()

    return __host_ips


import shutil

from BitTorrent.obsoletepythonsupport import set
from BitTorrent.TorrentPolicy import Policy
from BitTorrent.Torrent import *


class TorrentButler(Policy):

    def __init__(self, multitorrent):
        Policy.__init__(self, multitorrent)
        self.multitorrent = multitorrent


class EverythingAllOfTheTimeTorrentButler(TorrentButler):

    def __init__(self, multitorrent):
        TorrentButler.__init__(self, multitorrent)

    def butle(self):
        for torrent in self.multitorrent.get_torrents():
            if not self.multitorrent.torrent_running(torrent.metainfo.infohash):
                self.multitorrent.start_torrent(torrent.metainfo.infohash)



class EverythingOneTimeTorrentButler(TorrentButler):

    def __init__(self, multitorrent):
        TorrentButler.__init__(self, multitorrent)
        self.started_torrents = set()

    def butle(self):
        for torrent in self.multitorrent.get_torrents():
            if ((not self.multitorrent.torrent_running(torrent.metainfo.infohash)) and
                (not torrent.metainfo.infohash in self.started_torrents)):
                self.multitorrent.start_torrent(torrent.metainfo.infohash)
                self.started_torrents.add(torrent.metainfo.infohash)


class DownloadTorrentButler(TorrentButler):
    # TODO: this one should probably be configurable, once the new choker works
    SIMULTANEOUS = 3
    GOOD_RATE_THRESHOLD = 0.25  # minimal fraction of the average rate
    PURGATORY_TIME = 60  # seconds before we declare an underperformer "bad"
    LIMIT_MARGIN = 0.9  # consider bandwidth maximized at limit * margin
    REFRACTORY_PERIOD = 30  # seconds we wait after falling below the limit

    def __init__(self, multitorrent):
        TorrentButler.__init__(self, multitorrent)
        self.suspects = {}
        self.time_last_pushed_limit = bttime() - self.REFRACTORY_PERIOD

    def butles(self, torrent):
        return torrent.policy == "auto" and (not torrent.completed) and torrent.state in ["initialized", "running"]

    def butle(self):
        going = []
        waiting = []
        finishing = []
        num_initializing = 0
        for torrent in self.multitorrent.get_torrents():
            if self.butles(torrent):
                if self.multitorrent.torrent_running(torrent.metainfo.infohash):
                    going.append(torrent)
                elif torrent.is_initialized():
                    if torrent.get_percent_complete() < 1.0:
                        waiting.append(torrent)
                    else:
                        finishing.append(torrent)
            elif not torrent.completed:
                if torrent.state in ["created", "initializing"]:
                    if (bttime() - torrent.time_created < 10):
                        num_initializing += 1


        for t in finishing:
            self.multitorrent.start_torrent(t.metainfo.infohash)


        starting = [t for t in going
                    if bttime() - t.time_started < 60]

        transferring = [t for t in going
                        if t not in starting
                        and t.get_downrate() > 0.0
                        and t.get_num_connections() > 0]
        if (len(transferring) > 0):
            total_rate = sum([t.get_downrate() for t in transferring])
            good_rate = (total_rate / len(transferring)) * self.GOOD_RATE_THRESHOLD

            bad = []
            for t in transferring:
                if t.get_downrate() >= good_rate:
                    if self.suspects.has_key(t.metainfo.infohash):
                        #print t.working_path + " is good now, popping"
                        self.suspects.pop(t.metainfo.infohash)
                else:
                    if self.suspects.has_key(t.metainfo.infohash):
                        #print t.working_path, bttime() - self.suspects[t.metainfo.infohash]
                        if (bttime() - self.suspects[t.metainfo.infohash] >=
                            self.PURGATORY_TIME):
                            bad.append(t)
                    else:
                        #print t.working_path + " is bad now, inserting"
                        self.suspects[t.metainfo.infohash] = bttime()

            total_bad_rate = sum([t.get_downrate() for t in bad])
            num_virtual_good = total_bad_rate / good_rate
            num_good = len(transferring) - len(bad)
        else:
            num_good = 0
            num_virtual_good = 0

        uprate, downrate = self.multitorrent.get_total_rates()
        downrate_limit = self.multitorrent.config['max_download_rate']
        #print num_initializing, num_good, num_virtual_good, len(starting)
        #print downrate, '/', downrate_limit

        if (downrate >= downrate_limit * self.LIMIT_MARGIN):
            self.time_last_pushed_limit = bttime()
            #print "pushing limit"

        #print bttime() - self.time_last_pushed_limit
        if ((bttime() - self.time_last_pushed_limit >
             self.REFRACTORY_PERIOD) and
            (num_initializing == 0) and
            (num_good + num_virtual_good + len(starting) < self.SIMULTANEOUS)):
            high = []
            norm = []
            low = []
            for torrent in waiting:
                if torrent.priority == "high":
                    high.append(torrent)
                elif torrent.priority == "normal":
                    norm.append(torrent)
                elif torrent.priority == "low":
                    low.append(torrent)

            for p in (high, norm, low):
                best = None
                for torrent in p:
                    if ((not best) or
                        (best.get_percent_complete() == 0 and
                         torrent.total_bytes < best.total_bytes) or
                        (torrent.get_percent_complete() >
                         best.get_percent_complete())):
                        best = torrent
                if best:
                    break

            if best:
                self.multitorrent.start_torrent(best.metainfo.infohash)


class SeedTorrentButler(TorrentButler):

    FREQUENCY = 15
    MIN_RATE = 2000.0
    THRESHOLD = 1.25
    MIN_TRANSFERRING = 1

    def __init__(self, multitorrent):
        TorrentButler.__init__(self, multitorrent)
        self.counter = 0

    def butles(self, torrent):
        return torrent.policy == "auto" and torrent.completed and torrent.state in ["initialized", "running"]

    def butle(self):
        self.counter += 1
        self.counter %= self.FREQUENCY
        if self.counter != 0:
            return

        num_connections = 0
        transferring = []
        stopped = []
        total_rate = 0.0
        for torrent in self.multitorrent.get_torrents():
            if self.butles(torrent):
                if self.multitorrent.torrent_running(torrent.metainfo.infohash):
                    #print "found running torrent: ", torrent.get_uprate(), torrent.get_num_connections(), torrent._activity
                    if (torrent.get_uprate() > 0.0
                        and torrent.get_num_connections() > 0):
                        transferring.append(torrent)
                        for c in torrent.get_connections():
                            total_rate += c.upload.measure.get_rate()
                            num_connections += 1
                else:
                    #print "found stopped torrent: ", torrent._activity
                    stopped.append(torrent)

        #print num_connections, len(transferring), len(stopped)

        if (len(transferring) < self.MIN_TRANSFERRING
            or total_rate / num_connections > self.MIN_RATE * self.THRESHOLD):
            if len(stopped):
                r = random.randint(0, len(stopped) - 1)
                #print "starting torrent"
                self.multitorrent.start_torrent(stopped[r].metainfo.infohash)
        elif total_rate / num_connections < self.MIN_RATE:
            if len(transferring) > self.MIN_TRANSFERRING:

                def lambda_dammit(x, y):
                    try:
                        x_contribution = x.get_uprate() / x.get_avg_peer_downrate()
                    except ZeroDivisionError:
                        x_contribution = 1.0

                    try:
                        y_contribution = y.get_uprate() / y.get_avg_peer_downrate()
                    except ZeroDivisionError:
                        y_contribution = 1.0

                    return cmp(x_contribution, y_contribution)

                transferring.sort(lambda_dammit)
                self.multitorrent.stop_torrent(transferring[0].metainfo.infohash)




# threads are dumb, this module is smart.
#
# by Greg Hazel

import sys
import time
import atexit
import threading

def _get_non_daemons():
    return [d for d in threading.enumerate() if not d.isDaemon() and d != threading.currentThread()]

def register(func, *targs, **kargs):
    def duh():
        nondaemons = _get_non_daemons()
        for th in nondaemons:
            th.join()
        func(*targs, **kargs)
    atexit.register(duh)


def megadeth():
    time.sleep(10)
    try:
        import wx
        wx.Kill(wx.GetProcessId(), wx.SIGKILL)
    except:
        pass

def register_verbose(func, *targs, **kargs):
    def duh():
        nondaemons = _get_non_daemons()
        timeout = 4
        for th in nondaemons:
            start = time.time()
            th.join(timeout)
            timeout = max(0, timeout - (time.time() - start))
            if timeout == 0:
                break

        # kill all the losers
        # remove this when there are no more losers
        t = threading.Thread(target=megadeth)
        t.setDaemon(True)
        t.start()

        if timeout == 0:
            sys.stderr.write("non-daemon threads not shutting down "
                             "in a timely fashion:\n")
            nondaemons = _get_non_daemons()
            for th in nondaemons:
                sys.stderr.write("  %s\n" % th)
            sys.stderr.write("You have no chance to survive make your time.\n")
            for th in nondaemons:
                th.join()

        func(*targs, **kargs)

    atexit.register(duh)




# The contents of this file are subject to the BitTorrent Open Source License
# Version 1.1 (the License).  You may not copy or use this file, in either
# source code or executable form, except in compliance with the License.  You
# may obtain a copy of the License at http://www.bittorrent.com/license/.
#
# Software distributed under the License is distributed on an AS IS basis,
# WITHOUT WARRANTY OF ANY KIND, either express or implied.  See the License
# for the specific language governing rights and limitations under the
# License.

INTERVAL = 60
PERIODS = 5

class Hammerlock:
    def __init__(self, rate, call_later):
        self.rate = rate
        self.call_later = call_later
        self.curr = 0
        self.buckets = [{} for x in range(PERIODS)]
        self.call_later(INTERVAL, self._cycle)
        
    def _cycle(self):
        self.curr = (self.curr + 1) % PERIODS
        self.buckets[self.curr] = {}
        self.call_later(INTERVAL, self._cycle)

    def check(self, addr):
        x = self.buckets[self.curr].get(addr, 0) + 1
        self.buckets[self.curr][addr] = x
        x = 0
        for bucket in self.buckets:
            x += bucket.get(addr, 0) 
        if x >= self.rate:
            return False
        else:
            return True
    



# The contents of this file are subject to the BitTorrent Open Source License
# Version 1.0 (the License).  You may not copy or use this file, in either
# source code or executable form, except in compliance with the License.  You
# may obtain a copy of the License at http://www.bittorrent.com/license/.
#
# Software distributed under the License is distributed on an AS IS basis,
# WITHOUT WARRANTY OF ANY KIND, either express or implied.  See the License
# for the specific language governing rights and limitations under the
# License.

# Written by Matt Chisholm

import os
from BitTorrent.hash import sha
from time import time
try:
    getpid = os.getpid
except AttributeError:
    def getpid():
        return 1

from BitTorrent import version

def make_id():
    myid = 'M' + version.split()[0].replace('.', '-')
    padded = myid[:8] + '-' * (8-len(myid))
    myid = padded + sha(repr(time()) + ' ' +
                        str(getpid())).digest()[-6:].encode('hex')
    return myid



# The contents of this file are subject to the BitTorrent Open Source License
# Version 1.0 (the License).  You may not copy or use this file, in either
# source code or executable form, except in compliance with the License.  You
# may obtain a copy of the License at http://www.bittorrent.com/license/.
#
# Software distributed under the License is distributed on an AS IS basis,
# WITHOUT WARRANTY OF ANY KIND, either express or implied.  See the License
# for the specific language governing rights and limitations under the
# License.

# Author: Steve Hazel, Bram Cohen, and Uoti Urpala.

import os
import sys
import shutil
import socket
import cPickle
import logging
import traceback
from copy import copy
from BitTorrent.translation import _
from BitTorrent import GetTorrent
from BitTorrent.Choker import Choker
from BitTorrent.platform import bttime, encode_for_filesystem, old_broken_config_subencoding, get_filesystem_encoding
from BitTorrent.Torrent import Feedback, Torrent
from BitTorrent.bencode import bencode, bdecode
from BitTorrent.ConvertedMetainfo import ConvertedMetainfo
from BitTorrent.prefs import Preferences
from BitTorrent.NatTraversal import NatTraverser
from BitTorrent.BandwidthManager import BandwidthManager
from BitTorrent.InternetWatcher import InternetWatcher
from BitTorrent.Rerequester import Rerequester, DHTRerequester
from BitTorrent.NewRateLimiter import MultiRateLimiter as RateLimiter
from BitTorrent.ConnectionManager import SingleportListener
from BitTorrent.CurrentRateMeasure import Measure
from BitTorrent.Storage import FilePool
from BitTorrent.yielddefer import launch_coroutine, _wrap_task
from BitTorrent.defer import Deferred, DeferredEvent
from BitTorrent import BTFailure, InfoHashType
from BitTorrent import configfile
import BitTorrent
from khashmir.utkhashmir import UTKhashmir

class TorrentException(BTFailure):
    pass
class TorrentAlreadyInQueue(TorrentException):
    pass
class TorrentAlreadyRunning(TorrentException):
    pass
class TorrentNotInitialized(TorrentException):
    pass
class TorrentNotRunning(TorrentException):
    pass
class UnknownInfohash(TorrentException):
    pass
class TorrentShutdownFailed(TorrentException):
    pass
class TooManyTorrents(TorrentException):
    pass

#class DummyTorrent(object):
#    def __init__(self, infohash):
#        self.metainfo = object()
#        self.metainfo.infohash = infohash

BUTLE_INTERVAL = 1

class MultiTorrent(Feedback):
    """A MultiTorrent object represents a set of BitTorrent file transfers.
       It acts as a factory for Torrent objects, and it acts as
       the interface through which communication is performed to and from
       torrent file transfers.

       If you wish to instantiate MultiTorrent to download only a single
       torrent then pass is_single_torrent=True.

       """

    def __init__(self, config, rawserver,
                 data_dir, listen_fail_ok=False, init_torrents=True,
                 is_single_torrent=False):
        """
         @param config: program-wide configuration object.
         @param rawserver: object that manages main event loop and event
           scheduling.
         @param data_dir:  where variable data such as fastresume information
           and GUI state is saved.
         @param listen_fail_ok: if false, a BTFailure is raised if
           a server socket cannot be opened to accept incoming peer
           connections.
         @param init_torrents: restore state from prior instantiations of
           MultiTorrent.
         @param is_single_torrent: if true then allow only one torrent
           at a time in this MultiTorrent.
        """
        # is_single_torrent will go away when we move MultiTorrent into
        # a separate process, in which case, single torrent applications like
        # curses and console will act as a client to the MultiTorrent daemon.
        #   --Dave
        assert isinstance(config, Preferences)
        #assert isinstance(data_dir, unicode)  # temporarily commented -Dave
        assert isinstance(listen_fail_ok, bool)

        self.config = config
        self.data_dir = data_dir
        self.last_save_time = 0
        self.policies = []
        self.torrents = {}
        self.running = {}
        self.log_root = "core.MultiTorrent"
        self.logger = logging.getLogger(self.log_root)
        self.is_single_torrent = is_single_torrent

        self.auto_update_policy_index = None

        self.dht = None
        self.rawserver = rawserver
        nattraverser = NatTraverser(self.rawserver)
        self.internet_watcher = InternetWatcher(self.rawserver)
        self.singleport_listener = SingleportListener(self.rawserver,
                                                      nattraverser,
                                                      self.log_root)
        self.choker = Choker(self.config, self.rawserver.add_task)
        self.ratelimiter = RateLimiter(self.rawserver.add_task)
        self.ratelimiter.set_parameters(config['max_upload_rate'],
                                        config['upload_unit_size'])
        self.total_downmeasure = Measure(config['max_rate_period'])

        self._find_port(listen_fail_ok)

        self.filepool_doneflag = DeferredEvent()
        self.filepool = FilePool(self.filepool_doneflag,
                                 self.rawserver.add_task,
                                 self.rawserver.external_add_task,
                                 config['max_files_open'],
                                 config['num_disk_threads'])

        if not is_single_torrent:
            try:
                self._restore_state(init_torrents)
            except BTFailure:
                # don't be retarted.
                self.logger.exception("_restore_state failed")

        def no_dump_set_option(option, value):
            self.set_option(option, value, dump=False)

        self.bandwidth_manager = BandwidthManager(
            self.rawserver.external_add_task, config,
            no_dump_set_option, self.rawserver.get_remote_endpoints,
            get_rates=self.get_total_rates )

        self.rawserver.add_task(0, self.butle)
        #raise Exception("blah")


    def butle(self):
        policy = None
        try:
            for policy in self.policies:
                policy.butle()
        except:
            # You had something to hide, should have hidden it shouldn't you?
            self.logger.error("Butler error", exc_info=sys.exc_info())
            # Should we remove policies?
            #if policy:
            #    self.policies.remove(policy)
        self.rawserver.add_task(BUTLE_INTERVAL, self.butle)


    def _find_port(self, listen_fail_ok=True):
        """Run BitTorrent on the first available port found starting
           from minport in the range [minport, maxport]."""

        exc_info = None

        self.config['minport'] = max(1024, self.config['minport'])

        self.config['maxport'] = max(self.config['minport'],
                                     self.config['maxport'])

        e = (_("maxport less than minport - no ports to check") +
             (": %s %s" % (self.config['minport'], self.config['maxport'])))

        for port in xrange(self.config['minport'], self.config['maxport'] + 1):
            try:
                self.singleport_listener.open_port(port, self.config)
                if self.config['start_trackerless_client']:
                    self.dht = UTKhashmir(self.config['bind'],
                                   self.singleport_listener.get_port(),
                                   self.data_dir, self.rawserver,
                                   int(self.config['max_upload_rate'] * 0.01),
                                   rlcount=self.ratelimiter.increase_offset,
                                   config=self.config)
                break
            except socket.error, e:
                exc_info = sys.exc_info()
        else:
            if not listen_fail_ok:
                raise BTFailure, (_("Could not open a listening port: %s.") %
                                  unicode(e.args[0]) )
            self.global_error(logging.CRITICAL,
                              (_("Could not open a listening port: %s. ") % e) +
                              (_("Check your port range settings (%s:%s-%s).") %
                               (self.config['bind'], self.config['minport'],
                                self.config['maxport'])),
                              exc_info=exc_info)

    def shutdown(self):
        df = launch_coroutine(_wrap_task(self.rawserver.add_task), self._shutdown)
        df.addErrback(lambda e : self.logger.error('shutdown failed!',
                                                   exc_info=e))
        return df

    def _shutdown(self):
        self.choker.shutdown()
        self.singleport_listener.close_sockets()
        for t in self.torrents.itervalues():
            try:
                df = t.shutdown()
                yield df
                df.getResult()
                totals = t.get_total_transfer()
                t.uptotal = t.uptotal_old + totals[0]
                t.downtotal = t.downtotal_old + totals[1]
            except:
                t.logger.debug("Torrent shutdown failed in state: %s", t.state)
                print "Torrent shutdown failed in state:", t.state
                traceback.print_exc()

        # the filepool must be shut down after the torrents,
        # or pending ops could never complete
        self.filepool_doneflag.set()

        # hmm
        self._dump_torrents()


    def set_option(self, option, value, infohash=None, dump=True):
        if infohash is not None:
            t = self.get_torrent(infohash)
            t.config[option] = value
            if dump:
                t._dump_torrent_config()
        else:
            self.config[option] = value
            if dump:
                self._dump_global_config()

        if option in ['max_upload_rate', 'upload_unit_size']:
            self.ratelimiter.set_parameters(self.config['max_upload_rate'],
                                            self.config['upload_unit_size'])
        elif option == 'max_download_rate':
            pass # polled from the config automatically by MultiDownload
        elif option == 'max_files_open':
            self.filepool.set_max_files_open(value)
        elif option == 'maxport':
            if not self.config['minport'] <= self.singleport_listener.port <= \
                   self.config['maxport']:
                self._find_port()

    def add_policy(self, policy):
        self.policies.append(policy)

    def add_auto_update_policy(self, policy):
        self.add_policy(policy)
        self.auto_update_policy_index = self.policies.index(policy)

    def global_error(self, severity, message, exc_info=None):
        self.logger.log(severity, message, exc_info=exc_info)

    def create_torrent(self, metainfo, save_incomplete_as, save_as,
                       hidden=False, is_auto_update=False):
        if self.is_single_torrent and len(self.torrents) > 0:
            print "create_torrent: len(self.torrents)=", len(self.torrents)
            raise TooManyTorrents(_("MultiTorrent is set to download only "
                 "a single torrent, but tried to create more than one."))

        #save_as, junk = encode_for_filesystem(save_as)
        #save_incomplete_as, junk = encode_for_filesystem(save_incomplete_as)
        infohash = metainfo.infohash
        if self.torrent_known(infohash):
            if self.torrent_running(infohash):
                msg = _("This torrent (or one with the same contents) is "
                        "already running.")
                raise TorrentAlreadyRunning(msg)
            else:
                raise TorrentAlreadyInQueue(_("This torrent (or one with the same contents) is "
                                              "already waiting to run."))
        self._dump_metainfo(metainfo)

        #BUG.  Use _read_torrent_config for 5.0?  --Dave
        config = configfile.read_torrent_config(self.config,
                                                self.data_dir,
                                                infohash,
                                                lambda s : self.global_error(logging.ERROR, s))

        t = Torrent(metainfo, save_incomplete_as, save_as, self.config,
                    self.data_dir, self.rawserver, self.choker,
                    self.singleport_listener, self.ratelimiter,
                    self.total_downmeasure, self.filepool, self.dht, self,
                    self.log_root, hidden=hidden, is_auto_update=is_auto_update)
        retdf = Deferred()

        def torrent_started(*args):
            if config:
                t.update_config(config)

            t._dump_torrent_config()
            self._dump_torrents()
            t.metainfo.show_encoding_errors(self.logger.log)

            retdf.callback(t)

        df = self._init_torrent(t, use_policy=False)
        df.addCallback(torrent_started)

        return retdf


    def remove_torrent(self, ihash, del_files=False):
        # this feels redundant. the torrent will stop the download itself,
        # can't we accomplish the rest through a callback or something?
        if self.torrent_running(ihash):
            self.stop_torrent(ihash)

        t = self.torrents[ihash]

        # super carefully determine whether these are really incomplete files
        fs_save_incomplete_in, junk = encode_for_filesystem(
            self.config['save_incomplete_in']
            )
        inco = ((not t.completed) and
                (t.working_path != t.destination_path) and
                t.working_path.startswith(fs_save_incomplete_in))
        del_files = del_files and inco

        df = t.shutdown()

        df.addCallback(lambda *args: t.remove_state_files(del_files=del_files))
        try:
            del self.running[ihash]
        except:
            pass
        # give the torrent a blank feedback, so post-mortem errors don't
        # confuse multitorrent
        t.feedback = Feedback()
        del self.torrents[ihash]
        self._dump_torrents()

        return df


    def reinitialize_torrent(self, infohash):
        t = self.get_torrent(infohash)
        if self.torrent_running(infohash):
            assert t.is_running()
            raise TorrentAlreadyRunning(infohash.encode("hex"))
        assert t.state == "failed"

        df = self._init_torrent(t, use_policy=False)
        return df


    def start_torrent(self, infohash):
        if self.is_single_torrent and len(self.torrents) > 1:
            raise TooManyTorrents(_("MultiTorrent is set to download only "
                 "a single torrent, but tried to create more than one."))
        t = self.get_torrent(infohash)
        if self.torrent_running(infohash):
            assert t.is_running()
            raise TorrentAlreadyRunning(infohash.encode("hex"))
        if not t.is_initialized():
            raise TorrentNotInitialized(infohash.encode("hex"))

        t.logger.debug("starting torrent")

        self.running[infohash] = t
        t.start_download()
        t._dump_torrent_config()
        return t.state


    def stop_torrent(self, infohash, pause=False):
        if not self.torrent_running(infohash):
            raise TorrentNotRunning()
        t = self.get_torrent(infohash)
        assert t.is_running()

        t.logger.debug("stopping torrent")

        t.stop_download(pause=pause)
        del self.running[infohash]
        t._dump_torrent_config()
        return t.state

    def torrent_status(self, infohash, spew=False, fileinfo=False):
        torrent = self.get_torrent(infohash)
        status = torrent.get_status(spew, fileinfo)
        return torrent, status

    def get_torrent(self, infohash):
        try:
            t = self.torrents[infohash]
        except KeyError:
            raise UnknownInfohash(infohash.encode("hex"))
        return t

    def get_torrents(self):
        return self.torrents.values()

    def get_running(self):
        return self.running.keys()

    def get_visible_torrents(self):
        return [t for t in self.torrents.values() if not t.hidden]

    def get_visible_running(self):
        return [i for i in self.running.keys() if not self.torrents[i].hidden]

    def torrent_running(self, ihash):
        return ihash in self.running

    def torrent_known(self, ihash):
        return ihash in self.torrents

    def pause(self):
        for i in self.running.keys():
            self.stop_torrent(i, pause=True)

    def unpause(self):
        for i in [t.metainfo.infohash for t in self.torrents.values() if t.is_initialized()]:
            self.start_torrent(i)

    def set_file_priority(self, infohash, filename, priority):
        torrent = self.get_torrent(infohash)
        if torrent is None or not self.torrent_running(infohash):
            return
        torrent.set_file_priority(filename, priority)

    def set_torrent_priority(self, infohash, priority):
        torrent = self.get_torrent(infohash)
        if torrent is None:
            return
        torrent.priority = priority
        torrent._dump_torrent_config()

    def set_torrent_policy(self, infohash, policy):
        torrent = self.get_torrent(infohash)
        if torrent is None:
            return
        torrent.policy = policy
        torrent._dump_torrent_config()

    def get_all_rates(self):
        rates = {}
        for infohash, torrent in self.torrents.iteritems():
            rates[infohash] = (torrent.get_uprate() or 0,
                               torrent.get_downrate() or 0)
        return rates

    def get_variance(self):
        return self.bandwidth_manager.current_std, self.bandwidth_manager.max_std

    def get_total_rates(self):
        u = 0.0
        d = 0.0
        for torrent in self.torrents.itervalues():
            u += torrent.get_uprate() or 0
            d += torrent.get_downrate() or 0
        return u, d

    def get_total_totals(self):
        u = 0.0
        d = 0.0
        for torrent in self.torrents.itervalues():
            u += torrent.get_uptotal() or 0
            d += torrent.get_downtotal() or 0
        return u, d

    def auto_update_status(self):
        if self.auto_update_policy_index is not None:
            aub = self.policies[self.auto_update_policy_index]
            return aub.get_auto_update_status()
        return None, None, None

    def remove_auto_updates_except(self, infohash):
        for t in self.torrents.values():
            if t.is_auto_update and t.metainfo.infohash != infohash:
                self.remove_torrent(t.metainfo.infohash, del_files=True)


    ## singletorrent callbacks
    def started(self, torrent):
        torrent.logger.debug("started torrent")
        assert torrent.infohash in self.torrents
        torrent._dump_torrent_config()
        for policy in self.policies:
            policy.started(torrent)

    def failed(self, torrent):
        torrent.logger.debug("torrent failed")
        if torrent.infohash not in self.running:
            return
        del self.running[torrent.infohash]
        t = self.get_torrent(torrent.infohash)
        for policy in self.policies:
            policy.failed(t)

    def finishing(self, torrent):
        torrent.logger.debug("torrent finishing")
        t = self.get_torrent(torrent.infohash)

    def finished(self, torrent):
        torrent.logger.debug("torrent finished")
        t = self.get_torrent(torrent.infohash)
        t._dump_torrent_config()
        for policy in self.policies:
            policy.finished(t)

    def exception(self, torrent, text):
        torrent.logger.debug("torrent threw exception: " + text)
        if torrent.infohash not in self.torrents:
            return
        for policy in self.policies:
            policy.exception(torrent, text)

    def error(self, torrent, level, text):
        torrent.logger.log(level, text)
        if torrent.infohash not in self.torrents:
            return
        for policy in self.policies:
            policy.error(torrent, level, text)


    ### persistence

    ## These should be the .torrent file!
    #################
    def _dump_metainfo(self, metainfo):
        infohash = metainfo.infohash
        path = os.path.join(self.data_dir, 'metainfo',
                            infohash.encode('hex'))

        f = file(path+'.new', 'wb')
        f.write(metainfo.to_data())
        f.close()
        shutil.move(path+'.new', path)

    def _read_metainfo(self, infohash):
        path = os.path.join(self.data_dir, 'metainfo',
                            infohash.encode('hex'))
        f = file(path, 'rb')
        data = f.read()
        f.close()
        return ConvertedMetainfo(bdecode(data))
    #################

    def _read_torrent_config(self, infohash):
        path = os.path.join(self.data_dir, 'torrents', infohash.encode('hex'))
        if not os.path.exists(path):
            raise BTFailure,_("Coult not open the torrent config: " + infohash.encode('hex'))
        f = file(path, 'rb')
        data = f.read()
        f.close()
        try:
            torrent_config = cPickle.loads(data)
        except:
            # backward compatibility with <= 4.9.3
            torrent_config = bdecode(data)
            for k, v in torrent_config.iteritems():
                try:
                    torrent_config[k] = v.decode('utf8')
                    if k in ('destination_path', 'working_path'):
                        torrent_config[k] = encode_for_filesystem(torrent_config[k])[0]
                except:
                    pass
        if not torrent_config.get('destination_path'):
            raise BTFailure( _("Invalid torrent config file"))
        if not torrent_config.get('working_path'):
            raise BTFailure( _("Invalid torrent config file"))

        if get_filesystem_encoding() == None:
            # These paths should both be unicode.  If they aren't, they are the
            # broken product of some old version, and probably are in the
            # encoding we used to use in config files.  Attempt to recover.
            dp = torrent_config['destination_path']
            if isinstance(dp, str):
                try:
                    dp = dp.decode(old_broken_config_subencoding)
                    torrent_config['destination_path'] = dp
                except:
                    raise BTFailure( _("Invalid torrent config file"))

            wp = torrent_config['working_path']
            if isinstance(wp, str):
                try:
                    wp = wp.decode(old_broken_config_subencoding)
                    torrent_config['working_path'] = wp
                except:
                    raise BTFailure( _("Invalid torrent config file"))

        return torrent_config

    def _dump_global_config(self):
        # BUG: we can save to different sections later
        section = 'bittorrent'
        configfile.save_global_config(self.config, section,
                                      lambda *e : self.logger.error(*e))

    def _dump_torrents(self):
        self.last_save_time = bttime()
        r = []
        def write_entry(infohash, t):
            r.append(' '.join((infohash.encode('hex'),
                               str(t.uptotal), str(t.downtotal))))
        r.append('BitTorrent UI state file, version 5')
        r.append('Queued torrents')
        for t in self.torrents.values():
            write_entry(t.metainfo.infohash, self.torrents[t.metainfo.infohash])
        r.append('End')
        f = None
        try:
            path = os.path.join(self.data_dir, 'ui_state')
            f = file(path+'.new', 'wb')
            f.write('\n'.join(r) + '\n')
            f.close()
            shutil.move(path+'.new', path)
        except Exception, e:
            self.logger.error(_("Could not save UI state: ") + unicode(e.args[0]))
            if f is not None:
                f.close()

    def _init_torrent(self, t, initialize=True, use_policy=True):
        self.torrents[t.infohash] = t
        if not initialize:
            t.logger.debug("created torrent")
            return
        t.logger.debug("created torrent, initializing")
        df = t.initialize()
        if use_policy and t.policy == "start":
            df.addCallback(lambda r, t: self.start_torrent(t.infohash), t)
        return df

    def initialize_torrents(self):
        df = launch_coroutine(_wrap_task(self.rawserver.add_task), self._initialize_torrents)
        df.addErrback(lambda e : self.logger.error('initialize_torrents failed!',
                                                   exc_info=e))
        return df

    def _initialize_torrents(self):
        self.logger.debug("initializing torrents")
        for t in copy(self.torrents).itervalues():
            if t in self.torrents.values() and t.state == "created":
                df = self._init_torrent(t)
                # HACK
                #yield df
                #df.getResult()

    # this function is so nasty!
    def _restore_state(self, init_torrents):
        def decode_line(line):
            hashtext = line[:40]
            try:
                infohash = InfoHashType(hashtext.decode('hex'))
            except:
                raise BTFailure(_("Invalid state file contents"))
            if len(infohash) != 20:
                raise BTFailure(_("Invalid state file contents"))
            if infohash in self.torrents:
                raise BTFailure(_("Invalid state file (duplicate entry)"))

            try:
                metainfo = self._read_metainfo(infohash)
            except OSError, e:
                try:
                    f.close()
                except:
                    pass
                self.logger.error((_("Error reading metainfo file \"%s\".") %
                                  hashtext) + " (" + unicode(e.args[0])+ "), " +
                                  _("cannot restore state completely"))
                return None
            except Exception, e:
                self.logger.error((_("Corrupt data in metainfo \"%s\", cannot restore torrent.") % hashtext) +
                                  '('+unicode(e.args[0])+')')
                return None

            t = Torrent(metainfo, "",  "", self.config, self.data_dir,
                        self.rawserver, self.choker,
                        self.singleport_listener, self.ratelimiter,
                        self.total_downmeasure, self.filepool, self.dht, self,
                        self.log_root)
            t.metainfo.reported_errors = True # suppress redisplay on restart
            if infohash != t.metainfo.infohash:
                self.logger.error((_("Corrupt data in \"%s\", cannot restore torrent.") % hashtext) +
                                  _("(infohash mismatch)"))
                return None
            if len(line) == 41:
                t.working_path = None
                t.destination_path = None
                return infohash, t
            try:
                if version < 2:
                    t.working_path = line[41:-1].decode('string_escape')
                    t.working_path = t.working_path.decode('utf-8')
                    t.working_path = encode_for_filesystem(t.working_path)[0]
                    t.destination_path = t.working_path
                elif version == 3:
                    up, down, working_path = line[41:-1].split(' ', 2)
                    t.uptotal = t.uptotal_old = int(up)
                    t.downtotal = t.downtotal_old = int(down)
                    t.working_path = working_path.decode('string_escape')
                    t.working_path = t.working_path.decode('utf-8')
                    t.working_path = encode_for_filesystem(t.working_path)[0]
                    t.destination_path = t.working_path
                elif version >= 4:
                    up, down = line[41:-1].split(' ', 1)
                    t.uptotal = t.uptotal_old = int(up)
                    t.downtotal = t.downtotal_old = int(down)
            except ValueError:  # unpack, int(), decode()
                raise BTFailure(_("Invalid state file (bad entry)"))

            torrent_config = self.config
            try:
                if version < 5:
                    torrent_config = configfile.read_torrent_config(
                                                           self.config,
                                                           self.data_dir,
                                                           infohash,
                                                           lambda s : self.global_error(logging.ERROR, s))
                else:
                    torrent_config = self._read_torrent_config(infohash)
                t.update_config(torrent_config)
            except BTFailure, e:
                self.logger.error(e)
                # if read_torrent_config fails then ignore the torrent...
                return None

            return infohash, t
        # BEGIN _restore_state
        assert not self.is_single_torrent
        filename = os.path.join(self.data_dir, 'ui_state')
        if not os.path.exists(filename):
            return
        f = None
        try:
            f = file(filename, 'rb')
            lines = f.readlines()
            f.close()
        except Exception, e:
            if f is not None:
                f.close()
            raise BTFailure(unicode(e.args[0]))
        i = iter(lines)
        try:
            txt = 'BitTorrent UI state file, version '
            version = i.next()
            if not version.startswith(txt):
                raise BTFailure(_("Bad UI state file"))
            try:
                version = int(version[len(txt):-1])
            except:
                raise BTFailure(_("Bad UI state file version"))
            if version > 5:
                raise BTFailure(_("Unsupported UI state file version (from "
                                  "newer client version?)"))
            if version < 3:
                if i.next() != 'Running/queued torrents\n':
                    raise BTFailure(_("Invalid state file contents"))
            else:
                if i.next() != 'Running torrents\n' and version != 5:
                    raise BTFailure(_("Invalid state file contents"))
                while version < 5:
                    line = i.next()
                    if line == 'Queued torrents\n':
                        break
                    t = decode_line(line)
                    if t is None:
                        continue
                    infohash, t = t
                    df = self._init_torrent(t, initialize=init_torrents)
            while True:
                line = i.next()
                if (version < 5 and line == 'Known torrents\n') or (version == 5 and line == 'End\n'):
                    break
                t = decode_line(line)
                if t is None:
                    continue
                infohash, t = t
                if t.destination_path is None:
                    raise BTFailure(_("Invalid state file contents"))
                df = self._init_torrent(t, initialize=init_torrents)

            while version < 5:
                line = i.next()
                if line == 'End\n':
                    break
                t = decode_line(line)
                if t is None:
                    continue
                infohash, t = t
                df = self._init_torrent(t, initialize=init_torrents)
        except StopIteration:
            raise BTFailure(_("Invalid state file contents"))





# The contents of this file are subject to the BitTorrent Open Source License
# Version 1.1 (the License).  You may not copy or use this file, in either
# source code or executable form, except in compliance with the License.  You
# may obtain a copy of the License at http://www.bittorrent.com/license/.
#
# Software distributed under the License is distributed on an AS IS basis,
# WITHOUT WARRANTY OF ANY KIND, either express or implied.  See the License
# for the specific language governing rights and limitations under the
# License.


# Needs redesign.  Too many modifications to add a ui.  Adding a UI
# can easily break existing UIs unintentionally. --Dave

import os
from BitTorrent.translation import _

### add your favorite here
BAD_LIBC_WORKAROUND_DEFAULT = False
if os.name == 'posix':
    if os.uname()[0] in ['Darwin']:
        BAD_LIBC_WORKAROUND_DEFAULT = True

MAX_INCOMPLETE = 100
MAX_FILES_OPEN = 50
if os.name == 'nt':
    import ctypes
    from BitTorrent.platform import win_version_num
    # starting in XP SP2 the incomplete outgoing connection limit was set to 10
    if win_version_num >= (2, 5, 1, 2, 0):
        MAX_INCOMPLETE = 10

    # try to set it as high as possible
    # technically 2048 is max, but I see 512 sometimes, and I think win98
    # defaults to 50. If we're not the last person to call it, I think we get
    # errors, so screw it for now.
    #ctypes.cdll.msvcrt._setmaxstdio(512)
    # -3 for stdin, stdout, and stderr
    # -15 for a buffer
    MAX_FILES_OPEN = ctypes.cdll.msvcrt._getmaxstdio() - 3 - 15

from BitTorrent import languages, app_name

basic_options = [
    ('data_dir', u'',
     _("directory under which variable data such as fastresume information "
       "and GUI state is saved. Defaults to subdirectory 'data' of the "
       "bittorrent config directory.")),
    ('language', '',
     _("ISO Language code to use") + ': ' + ', '.join(languages)),
    ('use_factory_defaults', False,
     _("Starts the application in a debug mode.  All settings revert to "
       "default except those provided as command-line options.  Creates "
       "temporary directories for dot, data, incomplete torrents and "
       "complete torrents.  Allows multiple clients on the same machine to "
       "communicate with each other." )),
    ]

common_options = [
    ('ip', '',
     _("ip to report to the tracker (has no effect unless you are on the same "
       "local network as the tracker)")),
    ('forwarded_port', 0,
     _("world-visible port number if it's different from the one the client "
       "listens on locally")),
    ('minport', 6881,
     _("minimum port to listen on, counts up if unavailable")),
    ('maxport', 6999,
     _("maximum port to listen on")),
    ('bind', '',
     _("ip to bind to locally")),
    ('display_interval', 1.0,
     _("seconds between updates of displayed information")),
    ('rerequest_interval', 5 * 60,
     _("minutes to wait between requesting more peers")),
    ('min_peers', 20,
     _("minimum number of peers to not do rerequesting")),
    ('max_initiate', 60,
     _("number of peers at which to stop initiating new connections")),
    ('max_incomplete', MAX_INCOMPLETE,
     _("max number of outgoing incomplete connections")),
    ('max_allow_in', 80,
     _("maximum number of connections to allow, after this new incoming "
       "connections will be immediately closed")),
    ('check_hashes', True,
     _("whether to check hashes on disk")),
    ('max_upload_rate', 125000000, # 1 GBit local net = 125MB/s
     _("maximum B/s to upload at")),
    ('max_download_rate', 125000000, # 1 GBit local net = 125MB/s
     _("average maximum B/s to download at")),
    ('bandwidth_management', os.name == 'nt',
     _("automatic bandwidth management (Windows only)")),
    ('min_uploads', 2,
     _("the number of uploads to fill out to with extra optimistic unchokes")),
    ('max_files_open', MAX_FILES_OPEN,
     _("the maximum number of files in a multifile torrent to keep open at a "
       "time, 0 means no limit. Used to avoid running out of file descriptors.")),
    ('start_trackerless_client', True,
     _("Initialize a trackerless client.  This must be enabled in order to download trackerless torrents.")),
    ('upnp', True,
     _("Enable automatic port mapping")+' (UPnP)'),
    ('xmlrpc_port', -1,
    _("Start the XMLRPC interface on the specified port. This "
      "XML-RPC-based RPC allows a remote program to control the client "
      "to enable automated hosting, conformance testing, and benchmarking.")),
    ]


rare_options = [
    ('keepalive_interval', 120.0,
     _("number of seconds to pause between sending keepalives")),
    ('download_slice_size', 2 ** 14,
     _("how many bytes to query for per request.")),
    ('max_message_length', 2 ** 23,
     _("maximum length prefix encoding you'll accept over the wire - larger "
       "values get the connection dropped.")),
    ('socket_timeout', 300.0,
     _("seconds to wait between closing sockets which nothing has been "
       "received on")),
    ('timeout_check_interval', 60.0,
     _("seconds to wait between checking if any connections have timed out")),
    ('max_slice_length', 32768,
     _("maximum length slice to send to peers, close connection if a larger "
       "request is received")),
    ('max_rate_period', 20.0,
     _("maximum time interval over which to estimate the current upload and download rates")),
    ('max_announce_retry_interval', 1800,
     _("maximum time to wait between retrying announces if they keep failing")),
    ('snub_time', 30.0,
     _("seconds to wait for data to come in over a connection before assuming "
       "it's semi-permanently choked")),
    ('rarest_first_cutoff', 4,
     _("number of downloads at which to switch from random to rarest first")),
    ('upload_unit_size', 1380,
     _("how many bytes to write into network buffers at once.")),
    ('retaliate_to_garbled_data', True,
     _("refuse further connections from addresses with broken or intentionally "
       "hostile peers that send incorrect data")),
    ('one_connection_per_ip', True,
     _("do not connect to several peers that have the same IP address")),
    ('one_download_per_torrent', True,
     _("do not allow simultaneous downloads of the same torrent.")),
    ('peer_socket_tos', 8,
     _("if nonzero, set the TOS option for peer connections to this value")),
    ('bad_libc_workaround', BAD_LIBC_WORKAROUND_DEFAULT,
     _("enable workaround for a bug in BSD libc that makes file reads very slow.")),
    ('tracker_proxy', '',
     _("address of HTTP proxy to use for tracker connections")),
    ('close_with_rst', 0,
     _("close connections with RST and avoid the TCP TIME_WAIT state")),
    ('num_disk_threads', 10,
     _("number of read threads to use in the storage object")),
    ('num_piece_checks', 2,
     _("number of simultaneous piece checks to run per torrent, set to a low number like 2 or 3")),
    ('twisted', -1,
     _("Use Twisted network libraries for network connections. 1 means use twisted, 0 means do not use twisted, -1 means autodetect, and prefer twisted")),
    ('num_fast', 10,
     _("Number of pieces allowed fast.")),
    # Future.
    #('stream_priority', 2,
    # _("Priority for pieces that are needed soon.")),
    ]


tracker_options = [
    ('port', 80,
     _("Port to listen on.")),
    ('dfile', "",
     _("file to store recent downloader info in")),
    ('bind', '',
     _("ip to bind to locally")),
    ('socket_timeout', 15,
     _("timeout for closing connections")),
    ('close_with_rst', 0,
     _("close connections with RST and avoid the TCP TIME_WAIT state")),
    ('save_dfile_interval', 5 * 60,
     _("seconds between saving dfile")),
    ('timeout_downloaders_interval', 45 * 60,
     _("seconds between expiring downloaders")),
    ('reannounce_interval', 30 * 60,
     _("seconds downloaders should wait between reannouncements")),
    ('response_size', 50,
     _("default number of peers to send an info message to if the "
       "client does not specify a number")),
    ('timeout_check_interval', 5,
     _("time to wait between checking if any connections have timed out")),
    ('nat_check', 3,
     _("how many times to check if a downloader is behind a NAT "
       "(0 = don't check)")),
    ('log_nat_checks', 0,
     _("whether to add entries to the log for nat-check results")),
    ('min_time_between_log_flushes', 3.0,
     _("minimum time it must have been since the last flush to do "
       "another one")),
    ('min_time_between_cache_refreshes', 600.0,
     _("minimum time in seconds before a cache is considered stale "
       "and is flushed")),
    ('allowed_dir', u'',
     _("only allow downloads for .torrents in this dir (and recursively in "
       "subdirectories of directories that have no .torrent files "
       "themselves). If set, torrents in this directory show up on "
       "infopage/scrape whether they have peers or not")),
    ('parse_dir_interval', 60,
     _("how often to rescan the torrent directory, in seconds")),
    ('allowed_controls', 0,
     _("allow special keys in torrents in the allowed_dir to affect "
       "tracker access")),
    ('hupmonitor', 0,
     _("whether to reopen the log file upon receipt of HUP signal")),
    ('show_infopage', 1,
     _("whether to display an info page when the tracker's root dir "
       "is loaded")),
    ('infopage_redirect', '',
     _("a URL to redirect the info page to")),
    ('show_names', 1,
     _("whether to display names from allowed dir")),
    ('favicon', '',
     _("file containing x-icon data to return when browser requests "
       "favicon.ico")),
    ('only_local_override_ip', 2,
     _("ignore the ip GET parameter from machines which aren't on "
       "local network IPs (0 = never, 1 = always, 2 = ignore if NAT "
       "checking is not enabled). HTTP proxy headers giving address "
       "of original client are treated the same as --ip.")),
    ('logfile', '',
     _("file to write the tracker logs, use - for stdout (default)")),
    ('allow_get', 0,
     _("use with allowed_dir; adds a /file?hash={hash} url that "
       "allows users to download the torrent file")),
    ('keep_dead', 0,
     _("keep dead torrents after they expire (so they still show up on your "
       "/scrape and web page). Only matters if allowed_dir is not set")),
    ('scrape_allowed', 'full',
     _("scrape access allowed (can be none, specific or full)")),
    ('max_give', 200,
     _("maximum number of peers to give with any one request")),
    ('twisted', -1,
     _("Use Twisted network libraries for network connections. 1 means use twisted, 0 means do not use twisted, -1 means autodetect, and prefer twisted")),
    ('pid', 'bittorrent-tracker.pid',
     "Path to PID file"),
    ('max_incomplete', MAX_INCOMPLETE,
     _("max number of outgoing incomplete connections")),
    ]


def get_defaults(ui):
    assert ui in ("bittorrent" , "bittorrent-curses", "bittorrent-console" ,
                  "maketorrent",                      "maketorrent-console",
                                 "launchmany-curses", "launchmany-console" ,
                                                      "bittorrent-tracker" ,
                  "test-client"
                  )
    r = []

    if ui == 'test-client':
        ui = 'bittorrent-test-client'

    if ui == "bittorrent-tracker":
        r.extend(tracker_options)
    elif ui.startswith('bittorrent') or ui.startswith('launchmany'):
        r.extend(common_options)

    if ui == 'bittorrent':
        r.extend([
            ('publish', '',
             _("path to the file that you are publishing (seeding).")),
            ('verbose', False,
             _("display verbose information in user interface")),
            ('debug', False,
             _("provide debugging tools in user interface")),
            ('pause', False,
             _("start downloader in paused state")),
            ('open_from', u'',
             'local directory to look in for .torrent files to open'),
            ('start_minimized', False,
             _("Start %s minimized")%app_name),
            ('force_start_minimized', False,
             _("Start %s minimized (but do not save that preference)")%app_name),
            ('confirm_quit', True,
             _("Confirm before quitting %s")%app_name),
            ('new_version', '',
             _("override the version provided by the http version check "
               "and enable version check debugging mode")),
            ('current_version', '',
             _("override the current version used in the version check "
               "and enable version check debugging mode")),

            # remember GUI state
            ('progressbar_style', 3,
             _("The style of progressbar to show.  0 means no progress "
               "bar.  1 is an ordinary progress bar.  2 is a progress "
               "bar that shows transferring, available and missing "
               "percentages as well.  3 is a piece bar which "
               "color-codes each piece in the torrent based on its "
               "availability.")),
            ('geometry', '',
             _("specify window size and position, in the format: "
               "WIDTHxHEIGHT+XOFFSET+YOFFSET")),
            ('start_maximized', False,
             _("Start %s maximized")%app_name),
            ('column_widths', {},
             _("Widths of columns in torrent list in main window")),
            ('column_order', ['name', 'progress', 'eta', 'drate',
                              'urate', 'peers', 'priority', 'state'],
             _("Order of columns in torrent list in main window")),
            ('enabled_columns', ['name', 'progress', 'eta', 'drate',
                                 'priority'],
             _("Enabled columns in torrent list in main window")),
            ('sort_column', 'name',
             _("Default sort column in torrent list in main window")),
            ('sort_ascending', True,
             _("Default sort order in torrent list in main window")),
            ('toolbar_text', True,
             _("Whether to show text on the toolbar or not")),
            ('toolbar_size', 24,
             _("Size in pixels of toolbar icons")),
            ('show_details', False,
             _("Show details panel on startup")),
            ('details_tab', 0,
             _("Which tab in the details panel to show by default")),
            ('splitter_height', 300,
             _("Height of the details splitter when it is enabled")),
            ('ask_for_save', True,
             _("whether or not to ask for a location to save downloaded "
               "files in")),
            ('max_upload_rate', 40960, # 40KB/s up
             _("maximum B/s to upload at")),
            ])

        if os.name == 'nt':
            r.extend([
                ('launch_on_startup', True,
                 _("Launch %s when Windows starts") % app_name),
                ('minimize_to_tray', True,
                 _("Minimize to the system tray")),
                ('close_to_tray', False,
                 _("Close to the system tray")),
                ('enforce_association', True,
                 _("Enforce .torrent file associations on startup")),
            ])


    if ui in ('bittorrent', 'maketorrent'):
        r.append(
            ('theme', 'default',
             _("Icon theme to use"))
            )

    if ui.startswith('bittorrent') and ui != "bittorrent-tracker":
        r.extend([
            ('max_uploads', -1,
             _("the maximum number of uploads to allow at once. -1 means a "
               "(hopefully) reasonable number based on --max_upload_rate. "
               "The automatic values are only sensible when running one "
               "torrent at a time.")),
            ('save_in', u'',
             _("local directory where the torrent contents will be saved. The "
               "file (single-file torrents) or directory (batch torrents) will "
               "be created under this directory using the default name "
               "specified in the .torrent file. See also --save_as.")),
            ('save_incomplete_in', u'',
             _("local directory where the incomplete torrent downloads will be "
               "stored until completion.  Upon completion, downloads will be "
               "moved to the directory specified by --save_in.")),
            ])

    if ui.startswith('launchmany'):
        r.extend([
            ('max_uploads', 6,
             _("the maximum number of uploads to allow at once. -1 means a "
               "(hopefully) reasonable number based on --max_upload_rate. The "
               "automatic values are only sensible when running one torrent at "
               "a time.")),
            ('save_in', u'',
             _("local directory where the torrents will be saved, using a "
               "name determined by --saveas_style. If this is left empty "
               "each torrent will be saved under the directory of the "
               "corresponding .torrent file")),
            ('parse_dir_interval', 60,
              _("how often to rescan the torrent directory, in seconds") ),
            ('launch_delay', 0,
             _("wait this many seconds after noticing a torrent before starting it, to avoid race with tracker")),
            ('saveas_style', 4,
              _("How to name torrent downloads: "
                "1: use name OF torrent file (minus .torrent);  "
                "2: use name encoded IN torrent file;  "
                "3: create a directory with name OF torrent file "
                "(minus .torrent) and save in that directory using name "
                "encoded IN torrent file;  "
                "4: if name OF torrent file (minus .torrent) and name "
                "encoded IN torrent file are identical, use that "
                "name (style 1/2), otherwise create an intermediate "
                "directory as in style 3;  "
                "CAUTION: options 1 and 2 have the ability to "
                "overwrite files without warning and may present "
                "security issues."
                ) ),
            ('display_path', ui == 'launchmany-console' and True or False,
              _("whether to display the full path or the torrent contents for "
                "each torrent") ),
            ])

    if ui.startswith('launchmany') or ui == 'maketorrent':
        r.append(
            ('torrent_dir', u'',
             _("directory to look for .torrent files (semi-recursive)")),)

    if ui in ('bittorrent-curses', 'bittorrent-console'):
        r.extend([
            ('save_as', u'',
             _("file name (for single-file torrents) or directory name (for "
               "batch torrents) to save the torrent as, overriding the "
               "default name in the torrent. See also --save_in")),
            ('spew', False,
             _("whether to display diagnostic info to stdout")),])

    if ui == 'bittorrent-console' :
        r.extend([
            ('display_interval', 5,
            _("seconds between updates of displayed information")),
            ] )
    elif ui.startswith('launchmany'):
        r.extend([
            ('display_interval', 60,
            _("seconds between updates of displayed information")),
            ] )
        
    if ui.startswith('maketorrent'):
        r.extend([
            ('title', '',
             _("optional human-readable title for entire .torrent")),
            ('comment', '',
             _("optional human-readable comment to put in .torrent")),
            ('piece_size_pow2', 0,
             _("which power of two to set the piece size to, "
               "0 means pick a good piece size")),
            ('tracker_name', '',
             _("default tracker name")),
            ('tracker_list', '', ''),
            ('use_tracker', True,
             _("if false then make a trackerless torrent, instead of "
               "announce URL, use reliable node in form of <ip>:<port> or an "
               "empty string to pull some nodes from your routing table")),
            ('verbose', False,
             _("display verbose information in user interface")),
            ('debug', False,
             _("provide debugging tools in user interface")),
            ])

    r.extend(basic_options)

    if (ui.startswith('bittorrent') or ui.startswith('launchmany')) \
           and ui != "bittorrent-tracker":
        r.extend(rare_options)

    return r



# The contents of this file are subject to the BitTorrent Open Source License
# Version 1.1 (the License).  You may not copy or use this file, in either
# source code or executable form, except in compliance with the License.  You
# may obtain a copy of the License at http://www.bittorrent.com/license/.
#
# Software distributed under the License is distributed on an AS IS basis,
# WITHOUT WARRANTY OF ANY KIND, either express or implied.  See the License
# for the specific language governing rights and limitations under the
# License.

# Written by Bram Cohen

from BitTorrent.platform import bttime


class RateMeasure(object):

    def __init__(self, left):
        self.start = None
        self.last = None
        self.rate = 0
        self.remaining = None
        self.left = left
        self.broke = False
        self.got_anything = False
        self.when_next_expected = bttime() + 5

    def data_came_in(self, amount):
        if not self.got_anything:
            self.got_anything = True
            self.start = bttime() - 2
            self.last = self.start
            self.left -= amount
            return
        self.update(bttime(), amount)

    def data_rejected(self, amount):
        self.left += amount

    def get_time_left(self):
        if not self.got_anything:
            return None
        t = bttime()
        if t - self.last > 15:
            self.update(t, 0)
        return self.remaining

    def get_size_left(self):
        return self.left

    def update(self, t, amount):
        self.left -= amount
        if t < self.when_next_expected and amount == 0:
            return
        try:
            self.rate = ((self.rate * (self.last - self.start)) + amount) / (t - self.start)
            self.last = t
            self.remaining = self.left / self.rate
            if self.start < self.last - self.remaining:
                self.start = self.last - self.remaining
        except ZeroDivisionError:
            self.remaining = None
        if self.broke and self.last - self.start < 20:
            self.start = self.last - 20
        if self.last - self.start > 20:
            self.broke = True
        self.when_next_expected = t + min((amount / max(self.rate, 0.0001)), 5)



# The contents of this file are subject to the BitTorrent Open Source License
# Version 1.1 (the License).  You may not copy or use this file, in either
# source code or executable form, except in compliance with the License.  You
# may obtain a copy of the License at http://www.bittorrent.com/license/.
#
# Software distributed under the License is distributed on an AS IS basis,
# WITHOUT WARRANTY OF ANY KIND, either express or implied.  See the License
# for the specific language governing rights and limitations under the
# License.

# written by Steven Hazel

from __future__ import division

import random
import wx

from BitTorrent.translation import _
from BitTorrent.UI import Rate
from BitTorrent.GUI_wx import SPACING, BTDialog, BTFrame, BTApp, BTPanel, HSizer, VSizer, gui_wrap, CustomWidgets
from BitTorrent.yielddefer import launch_coroutine
from BitTorrent.Lists import SizedList


class HistoryCollector(object):

    def __init__(self, span, interval):
        self.max_len = int(span / (interval / 1000))
        self.interval = interval
        self.upload_data = SizedList(self.max_len)
        self.download_data = SizedList(self.max_len)
        self.max_upload_rate = 0
        self.max_download_rate = 0
        self.variance = 0
        self.max_variance = 1
        self.viewer = None

    def update(self, upload_rate, download_rate,
               max_upload_rate, max_download_rate,
               variance, max_variance):
#         if (len(self.data) == 0):
#             self.data.append(random.random() * 500.0)
#         else:
#             self.data.append(self.data[-1] * ((random.random() * 0.04) + 0.98))

        self.upload_data.append(upload_rate or 0)
        self.download_data.append(download_rate or 0)
        self.max_upload_rate = max_upload_rate
        self.max_download_rate = max_download_rate
        self.variance = variance
        self.max_variance = max_variance
        if self.viewer:
            self.viewer()

class BandwidthGraphPanel(CustomWidgets.DoubleBufferedMixin, BTPanel):
    MIN_MAX_DATA = 1000.0  # minimum value for the top of the graph

    def __init__(self, parent, history):
        super(BTPanel, self).__init__(parent,
              style=wx.NO_FULL_REPAINT_ON_RESIZE)
        self.history = history
        self.history.viewer = self.update
        self.upload_rate = 0
        self.download_rate = 0
        self.SetBackgroundColour("#002000")
        self.max_label_width = None
        CustomWidgets.DoubleBufferedMixin.__init__(self)

    def draw_graph(self, dc, max_len, data, max_data, rect):
        x_div = max_len - 1
        y_div = max_data
        if (y_div == 0.0):
            y_div = 1

        last_datum = (None, None)
        for i, datum in enumerate(data):
            if (last_datum == (None, None)):
                last_datum = (i, datum)
            dc.DrawLine(rect.x + (((last_datum[0] + (max_len - len(data))) / x_div) * (rect.width - 1)),
                        rect.y + ((1.0 - (last_datum[1] / y_div)) * (rect.height - 1)),
                        rect.x + (((i + (max_len - len(data))) / x_div) * (rect.width - 1)),
                        rect.y + ((1.0 - (datum / y_div)) * (rect.height - 1)))
            last_datum = (i, datum)

    def draw_max_line(self, dc, data, max_data, rect, offset=0):
        y_div = float(max_data)
        pos = (1.0 - (data / y_div)) * (rect.height - 1)
        if pos >= 0 and pos < rect.height:
            dc.DrawLine(rect.x + offset, rect.y + pos,
                        rect.x + rect.width, rect.y + pos)

    def update(self, force=False):
        if not self.IsShown() and not force:
            return
        dc = wx.BufferedDC(wx.ClientDC(self), self.buffer)
        self.draw(dc, size = self.GetClientSize())

    def draw(self, dc, size):
        s_rect = wx.Rect(0, 0, size.width, size.height)
        elements = list(self.history.upload_data) + list(self.history.download_data)
        max_data = max(elements + [self.MIN_MAX_DATA])

        interval = self.history.interval / 1000
        seconds = self.history.max_len * interval
        time_label_text = "%d"%seconds + _(" seconds, ")
        time_label_text += str(interval) + _(" second interval")

        dr_label_text = _("Download rate")
        ur_label_text = _("Upload rate")

        text_color = wx.NamedColour("light gray")
        border_color = wx.NamedColour("gray")
        dr_color = wx.NamedColor("green")
        ur_color = wx.NamedColor("slate blue")

        dc.SetFont(wx.Font(8, wx.DEFAULT, wx.NORMAL, wx.NORMAL))
        dc.SetBackground(wx.Brush(self.GetBackgroundColour()))
        dc.Clear()


        if (self.max_label_width == None):
            self.max_label_width = dc.GetTextExtent(unicode(Rate(1000000.0)))[0]

        self.max_label_width = max(self.max_label_width,
                                   dc.GetTextExtent(unicode(Rate(max_data)))[0])

        top_label_height = dc.GetTextExtent(unicode(Rate(max_data)))[1]
        bottom_label_height = dc.GetTextExtent(unicode(Rate(0.0)))[1]

        time_label_width = dc.GetTextExtent(unicode(time_label_text))[0]
        time_label_height = dc.GetTextExtent(unicode(time_label_text))[1]

        dr_label_width = dc.GetTextExtent(unicode(dr_label_text))[0]
        dr_label_height = dc.GetTextExtent(unicode(dr_label_text))[1]

        ur_label_width = dc.GetTextExtent(unicode(ur_label_text))[0]
        ur_label_height = dc.GetTextExtent(unicode(ur_label_text))[1]

        label_spacer = 4
        b_spacer = 15

        legend_box_size = 10
        legend_guts_height = max((legend_box_size,
                                  ur_label_height,
                                  dr_label_height))
        legend_height = legend_guts_height + b_spacer


        x1 = b_spacer
        y1 = b_spacer
        x2 = max(x1, s_rect.GetRight() -
                 (label_spacer + self.max_label_width + label_spacer))
        y2 = max(y1 + top_label_height + SPACING + bottom_label_height,
                 s_rect.GetBottom() -
                 (label_spacer + time_label_height + label_spacer +
                  legend_height))
        b_rect = wx.RectPP(wx.Point(x1, y1),
                           wx.Point(x2, y2))

        x1 = b_spacer + b_spacer
        y1 = max((b_rect.GetBottom() +
                  label_spacer + time_label_height + label_spacer),
                 s_rect.GetBottom() - (legend_box_size + b_spacer))
        x2 = x1 + legend_box_size
        y2 = y1 + legend_box_size
        db_rect = wx.RectPP(wx.Point(x1, y1),
                            wx.Point(x2, y2))

        x1 = db_rect.GetRight() + label_spacer + dr_label_width + b_spacer
        y1 = db_rect.y
        x2 = x1 + legend_box_size
        y2 = y1 + legend_box_size
        ub_rect = wx.RectPP(wx.Point(x1, y1),
                            wx.Point(x2, y2))

        x1 = min(b_rect.x + 1, b_rect.GetRight())
        y1 = min(b_rect.y + 1, b_rect.GetBottom())
        x2 = max(x1, b_rect.GetRight() - 1)
        y2 = max(y1, b_rect.GetBottom() - 1)
        i_rect = wx.RectPP(wx.Point(x1, y1),
                           wx.Point(x2, y2))


        bw_label_x = b_rect.GetRight() + label_spacer
        time_label_x = max(b_rect.x,
                           (b_rect.GetRight() / 2) - (time_label_width / 2))


        dc.SetTextForeground(text_color)
        dc.DrawText(unicode(Rate(max_data)),
                    bw_label_x, b_rect.y)
        dc.DrawText(unicode(Rate(0.0)),
                    bw_label_x, b_rect.GetBottom() - bottom_label_height)
        dc.DrawText(unicode(time_label_text),
                    time_label_x,  b_rect.GetBottom() + label_spacer)
        dc.DrawText(unicode(dr_label_text),
                    db_rect.GetRight() + label_spacer,
                    db_rect.y + (legend_box_size / 2) - (dr_label_height / 2))
        dc.DrawText(unicode(ur_label_text),
                    ub_rect.GetRight() + label_spacer,
                    ub_rect.y + (legend_box_size / 2) - (ur_label_height / 2))

        pen = wx.Pen(border_color, 1, wx.SOLID)
        dc.SetPen(pen)

        brush = wx.Brush(dr_color)
        dc.SetBrush(brush)
        dc.DrawRectangle(db_rect.x, db_rect.y,
                         db_rect.GetWidth(), db_rect.GetHeight())

        brush = wx.Brush(ur_color)
        dc.SetBrush(brush)
        dc.DrawRectangle(ub_rect.x, ub_rect.y,
                         ub_rect.GetWidth(), ub_rect.GetHeight())

        dc.DrawLine(b_rect.x, b_rect.y,
                    b_rect.GetRight(), b_rect.y)
        dc.DrawLine(b_rect.x, b_rect.y,
                    b_rect.x, b_rect.GetBottom())
        dc.DrawLine(b_rect.x, b_rect.GetBottom(),
                    b_rect.GetRight(), b_rect.GetBottom())
        dc.DrawLine(b_rect.GetRight(), b_rect.y,
                    b_rect.GetRight(), b_rect.GetBottom())

        pen = wx.Pen(border_color, 1, wx.DOT)
        dc.SetPen(pen)
        dc.DrawLine(i_rect.x, i_rect.y + (i_rect.height * 0.75),
                    i_rect.GetRight(), i_rect.y + (i_rect.height * 0.75))
        dc.DrawLine(i_rect.x, i_rect.y + (i_rect.height * 0.5),
                    i_rect.GetRight(), i_rect.y + (i_rect.height * 0.5))
        dc.DrawLine(i_rect.x, i_rect.y + (i_rect.height * 0.25),
                    i_rect.GetRight(), i_rect.y + (i_rect.height * 0.25))

        pen = wx.Pen(ur_color, 1, wx.SHORT_DASH)
        dc.SetPen(pen)
        self.draw_max_line(dc, self.history.max_upload_rate, max_data, i_rect,
                           offset=3)
        pen = wx.Pen(ur_color, 1, wx.SOLID)
        dc.SetPen(pen)
        self.draw_graph(dc, self.history.max_len, self.history.upload_data,
                        max_data, i_rect)

        pen = wx.Pen(dr_color, 1, wx.SHORT_DASH)
        dc.SetPen(pen)
        self.draw_max_line(dc, self.history.max_download_rate, max_data, i_rect)
        pen = wx.Pen(dr_color, 1, wx.SOLID)
        dc.SetPen(pen)
        self.draw_graph(dc, self.history.max_len, self.history.download_data,
                        max_data, i_rect)

        ## variance line
        if False:
            pen = wx.Pen(wx.NamedColor("yellow"), 1, wx.SHORT_DASH)
            dc.SetPen(pen)
            self.draw_max_line(dc, self.history.variance, self.history.max_variance, i_rect)
        

class StatisticsPanel(wx.Panel):

    def __init__(self, parent, *a, **k):
        wx.Panel.__init__(self, parent, *a, **k)

        self.stats = {}

        self.sizer = wx.BoxSizer(wx.HORIZONTAL)
        self.scrolled_window = wx.ScrolledWindow(self)
        self.scrolled_window.SetScrollRate(1, 1)
        self.sizer.Add(self.scrolled_window, flag=wx.GROW, proportion=1)

        self.scroll_sizer = wx.BoxSizer(wx.HORIZONTAL)
        self.panel = wx.Panel(self.scrolled_window)
        self.scroll_sizer.Add(self.panel, flag=wx.GROW, proportion=1)

        self.outer = wx.BoxSizer(wx.HORIZONTAL)

        self.left = wx.GridSizer(6, 2, SPACING, SPACING)
        self.outer.Add(self.left, flag=wx.ALL, border=SPACING)

        self.right = wx.GridSizer(6, 2, SPACING, SPACING)
        self.outer.Add(self.right, flag=wx.ALL, border=SPACING)

        self.add_row(self.left, "total_downrate", _("Total Download Rate:"))
        self.add_row(self.left, "total_uprate", _("Total Upload Rate:"))

        self.add_blank_row(self.left)

        self.add_row(self.left, "total_downtotal", _("Total Downloaded:"))
        self.add_row(self.left, "total_uptotal", _("Total Uploaded:"))

        self.add_row(self.right, "num_torrents", _("Torrents:"))
        self.add_row(self.right, "num_running_torrents", _("Running Torrents:"))

        self.add_blank_row(self.right)

        self.add_row(self.right, "num_connections", _("Total Connections:"))
        self.add_row(self.right, "avg_connections", _("Connections per Torrent:"))

        self.panel.SetSizerAndFit(self.outer)
        self.scrolled_window.SetSizer(self.scroll_sizer)
        self.SetSizerAndFit(self.sizer)

        # this fixes background repaint issues on XP w/ themes
        def OnSize(event):
            self.Refresh()
            event.Skip()
        self.Bind(wx.EVT_SIZE, OnSize)

    def add_blank_row(self, sizer):
        sizer.AddSpacer((5, 5))
        sizer.AddSpacer((5, 5))


    def add_row(self, sizer, name, header, value=""):
        h = wx.StaticText(self.panel, label=header)
        f = h.GetFont()
        f.SetWeight(wx.FONTWEIGHT_BOLD)
        h.SetFont(f)
        sizer.Add(h)

        st = wx.StaticText(self.panel, label=value)
        sizer.Add(st)

        self.stats[name] = st


    def update_values(self, values):
        for name, st in self.stats.iteritems():
            if name not in values:
                continue
            s = unicode(values[name])
            if unicode(st.GetLabel()) != s:
                st.SetLabel(s)
        #self.left.Layout()
        #self.right.Layout()
        #self.outer.Layout()



class BlingWindow(BTFrame):
    def __init__(self, parent, history, *a, **k):
        super(BlingWindow, self).__init__(parent, title="Details",
            size=(640, 280),
            style=wx.DEFAULT_FRAME_STYLE|wx.NO_FULL_REPAINT_ON_RESIZE|wx.CLIP_CHILDREN)
        self.Bind(wx.EVT_CLOSE, self.close)
        self.bling = BandwidthGraphPanel(self, history)
        self.SetBackgroundColour(self.bling.GetBackgroundColour())
        self.sizer = wx.BoxSizer(wx.VERTICAL)
        self.sizer.Add(self.bling, flag=wx.GROW, proportion=1)
        self.SetSizer(self.sizer)

    def close(self, *e):
        self.Hide()


class BlingPanel(BTPanel):

    def __init__(self, parent, history, *a, **k):
        BTPanel.__init__(self, parent, *a, **k)
        #self.SetMinSize((200, 200))

        self.notebook = wx.Notebook(self, style=wx.CLIP_CHILDREN)

        self.statistics = StatisticsPanel(self.notebook, style=wx.CLIP_CHILDREN)
        self.notebook.AddPage(self.statistics, _("Statistics"))

        self.bling = BandwidthGraphPanel(self.notebook, history)
        self.speed_tab_index = self.notebook.GetPageCount()
        self.notebook.AddPage(self.bling, _("Speed"))

        self.notebook.Bind(wx.EVT_NOTEBOOK_PAGE_CHANGED, self.OnPageChanged)

        self.sizer.Add(self.notebook, flag=wx.GROW, proportion=1)

        self.Hide()
        self.sizer.Layout()

    def OnPageChanged(self, event):
        if event.GetSelection() == self.speed_tab_index:
            self.bling.update(force=True)
        event.Skip()



# The contents of this file are subject to the BitTorrent Open Source License
# Version 1.1 (the License).  You may not copy or use this file, in either
# source code or executable form, except in compliance with the License.  You
# may obtain a copy of the License at http://www.bittorrent.com/license/.
#
# Software distributed under the License is distributed on an AS IS basis,
# WITHOUT WARRANTY OF ANY KIND, either express or implied.  See the License
# for the specific language governing rights and limitations under the
# License.

# Written by Bram Cohen and Uoti Urpala

from __future__ import division
from __future__ import generators

import os
import gc
import sys
import time
import errno
import shutil
import random
import socket
import cPickle
import logging
import urlparse
import itertools
import traceback
from cStringIO import StringIO
from BitTorrent.hash import sha
from BitTorrent.translation import _
from BitTorrent.NamedMutex import NamedMutex

import BitTorrent.stackthreading as threading

from BitTorrent.platform import bttime, is_path_too_long
from BitTorrent.platform import decode_from_filesystem, get_filesystem_encoding
from BitTorrent.ConnectionManager import ConnectionManager
from BitTorrent import PeerID
from BitTorrent.defer import Deferred, ThreadedDeferred
from BitTorrent.yielddefer import launch_coroutine, _wrap_task
from BitTorrent.TorrentStats import TorrentStats
from BitTorrent.RateMeasure import RateMeasure
from BitTorrent.PiecePicker import PiecePicker
from BitTorrent.Rerequester import Rerequester, DHTRerequester
from BitTorrent.NewRateLimiter import MultiRateLimiter as RateLimiter
from BitTorrent.CurrentRateMeasure import Measure
from BitTorrent.Storage import Storage, FilePool
from BitTorrent.HTTPConnector import URLage
from BitTorrent.StorageWrapper import StorageWrapper
from BitTorrent.Upload import Upload
from BitTorrent.MultiDownload import MultiDownload
from BitTorrent import version
from BitTorrent import BTFailure, UserFailure
from BitTorrent.prefs import Preferences
#from BitTorrent.bencode import bencode


from khashmir import const


class Feedback(object):
    """Inidivual torrents (Torrent) perform callbacks regarding
       changes of state to the rest of the program via a Feedback
       object."""

    def finished(self, torrent):
        pass

    def failed(self, torrent):
        pass

    def error(self, torrent, level, text):
        pass

    def exception(self, torrent, text):
        self.error(torrent, logging.CRITICAL, text)

    def started(self, torrent):
        pass


class Torrent(object):
    """Represents a single file transfer or transfer for a batch of files
       in the case of a batch torrent.  During the course of a single
       transfer, a Torrent may have many different connections to peers."""

    STATES = ["created", "initializing", "initialized", "running",
              "finishing", "failed"]
    POLICIES = ["stop", "start", "auto"]
    PRIORITIES = ["low", "normal", "high"]

    def __init__(self, metainfo, working_path, destination_path, config,
                 data_dir, rawserver, choker,
                 singleport_listener, ratelimiter, total_downmeasure,
                 filepool, dht, feedback, log_root,
                 hidden=False, is_auto_update=False):
        # The passed working path and destination_path should be filesystem
        # encoded or should be unicode if the filesystem supports unicode.
        fs_encoding = get_filesystem_encoding()
        assert fs_encoding == None and isinstance(working_path,unicode) \
               or isinstance(working_path,str)
        assert fs_encoding == None and isinstance(destination_path,unicode) \
               or isinstance(destination_path,str)

        self.state = "created"
        self.data_dir = data_dir
        self.feedback = feedback
        self.finished_this_session = False
        self._rawserver = rawserver
        self._singleport_listener = singleport_listener
        self._ratelimiter = ratelimiter
        self._filepool = filepool
        self._dht = dht
        self._picker = None
        self._choker = choker
        self._storage = None
        self._storagewrapper = None
        self._ratemeasure = None
        self._upmeasure = None
        self._downmeasure = None
        self._total_downmeasure = total_downmeasure
        self._connection_manager = None
        self._rerequest = None
        self._statuscollector = None
        self._announced = False
        self._listening = False
        self.reserved_ports = []
        self.reported_port = None
        self._myfiles = None
        self._last_myfiles = None
        self.total_bytes = None
        self._doneflag = threading.Event()
        self.finflag = threading.Event()
        self._contfunc = None
        self._activity = (_("Initial startup"), 0)
        self._pending_file_priorities = []
        self._mutex = None
        self.time_created = bttime()
        self.time_started = None

        self.metainfo = metainfo
        self.infohash = metainfo.infohash

        self.log_root = log_root
        self.logger = logging.getLogger(log_root + '.' + repr(self.infohash))
        self.logger.setLevel(logging.DEBUG)

        self.total_bytes = metainfo.total_bytes
        if not metainfo.reported_errors:
            metainfo.show_encoding_errors(self._error)

        self.config = Preferences(config)#, persist_callback=self._dump_torrent_config)
        self.working_path = working_path #sets in config. See _set_working_path

        self.destination_path = destination_path # sets in config.
        self.priority = "normal"
        self.policy = "auto"

        self.hidden = hidden #sets in config
        self.is_auto_update = is_auto_update #sets in config

        self._completed = False
        self.config['finishtime'] = 0
        self.uptotal = 0
        self.uptotal_old = 0
        self.downtotal = 0
        self.downtotal_old = 0
        self.context_valid = True

    def update_config(self, config):
        self.config.update(config)

        d = self.config.get('file_priorities', {})
        for k, v in d.iteritems():
            self.set_file_priority(k, v)

        if self.policy not in self.POLICIES:
            self.policy = "auto"

        if self.priority not in self.PRIORITIES:
            self.priority = "normal"

    def _set_state(self, value):
        assert value in self.STATES
        self._state = value
    def _get_state(self):
        return self._state
    state = property(_get_state, _set_state)

    def _set_policy(self, value):
        assert value in self.POLICIES
        self.config['policy'] = value
    def _get_policy(self):
        return self.config['policy']
    policy = property(_get_policy, _set_policy)

    def _set_priority(self, value):
        assert value in self.PRIORITIES
        self.config['priority'] = value
    def _get_priority(self):
        return self.config['priority']
    priority = property(_get_priority, _set_priority)

    def _set_hidden(self, value):
        self.config['hidden'] = value
    def _get_hidden(self):
        return self.config['hidden']
    hidden = property(_get_hidden, _set_hidden)

    def _set_is_auto_update(self, value):
        self.config['is_auto_update'] = value
    def _get_is_auto_update(self):
        return self.config['is_auto_update']
    is_auto_update = property(_get_is_auto_update, _set_is_auto_update)

    def _set_completed(self, val):
        self._completed = val
        self.config['finishtime'] = bttime()
    def _get_completed(self):
        return self._completed
    completed = property(_get_completed, _set_completed)

    def _get_finishtime(self):
        return self.config['finishtime']
    finishtime = property(_get_finishtime)

    def _set_destination_path(self, value):
        # The following assertion will not always work.  Consider
        # Torrent.py: self.working_path = self.destination_path
        # This assignment retrieves a unicode path from
        # config['destination_path'].
        #assert isinstance(value,str) # assume filesystem encoding.
        #
        # The following if statement is not necessary because config here
        # is not really a config file, but rather state that is pickled when
        # the Torrent shuts down.
        #if isinstance(value, str):
        #    value = decode_from_filesystem(value)
        self.config['destination_path'] = value
    def _get_destination_path(self):
        return self.config['destination_path']
    destination_path = property(_get_destination_path, _set_destination_path)

    def _set_working_path(self, value):
        # See comments for _set_destination_path.
        self.config['working_path'] = value
    def _get_working_path(self):
        return self.config['working_path']
    working_path = property(_get_working_path, _set_working_path)

    def __cmp__(self, other):
        return cmp(self.metainfo.infohash, other.metainfo.infohash)

    def is_initialized(self):
        return self.state not in ["created", "initializing", "failed"]

    def is_running(self):
        return self.state == "running"

    def is_context_valid(self):
        return self.context_valid

    def _context_wrap(self, _f, *a, **kw):
        # this filters out calls
        # to an invalid torrent
        # sloppy technique
        if not self.context_valid:
            return
        try:
            _f(*a, **kw)
        except KeyboardInterrupt:
            raise
        except:
            self.got_exception(*sys.exc_info())

    # these wrappers add _context_wrap to the chain, so that calls on a dying
    # object are filtered, and errors on a valid call are logged.
    def add_task(self, delay, func, *a, **kw):
        self._rawserver.add_task(delay, self._context_wrap,
                                 func, *a, **kw)
    def external_add_task(self, delay, func, *a, **kw):
        self._rawserver.external_add_task(delay, self._context_wrap,
                                          func, *a, **kw)

    def _register_files(self):
        if self.metainfo.is_batch:
            myfiles = [os.path.join(self.destination_path, f) for f in
                       self.metainfo.files_fs]
        else:
            myfiles = [self.destination_path, ]

        for filename in myfiles:
            if is_path_too_long(filename):
                raise BTFailure("Filename path exceeds platform limit: %s" % filename)

        # if the destination path contains any of the files in the torrent
        # then use the destination path instead of the working path.
        if len([x for x in myfiles if os.path.exists(x)]) > 0:
            self.working_path = self.destination_path
        else:
            if self.metainfo.is_batch:
                myfiles = [os.path.join(self.working_path, f) for f in
                           self.metainfo.files_fs]
            else:
                myfiles = [self.working_path, ]

        assert self._myfiles == None, '_myfiles should be None!'
        self._filepool.add_files(myfiles, self)
        self._myfiles = myfiles

    def _build_url_mapping(self):
        # TODO: support non [-1] == '/' urls
        url_suffixes = []

        if self.metainfo.is_batch:
            for filename in self.metainfo.orig_files:
                path = '%s/%s' % (self.metainfo.name, filename)
                # am I right that orig_files could have windows paths?
                path = path.replace('\\', '/')
                url_suffixes.append(path)
        else:
            url_suffixes = [self.metainfo.name, ]
        self._url_suffixes = url_suffixes

        total = 0
        piece_size = self.metainfo.piece_length
        self._urls = zip(self._url_suffixes, self.metainfo.sizes)

    def _unregister_files(self):
        if self._myfiles is not None:
            self._filepool.remove_files(self._myfiles)
            self._last_myfiles = self._myfiles
            self._myfiles = None

    def initialize(self):
        self.context_valid = True
        assert self.state in ["created", "failed", "finishing"]
        self.state = "initializing"
        df = launch_coroutine(_wrap_task(self.add_task), self._initialize)
        df.addErrback(lambda e : self.got_exception(*e))
        return df

    # this function is so nasty!
    def _initialize(self):

        self._doneflag = threading.Event()

        # only one torrent object for of a particular infohash at a time.
        # Note: This must be done after doneflag is created if shutdown()
        # is to be called from got_exception().
        if self.config["one_download_per_torrent"]:
            self._mutex = NamedMutex(self.infohash.encode("hex"))
            if not self._mutex.acquire(False):
               try:
                   raise UserFailure(_("Torrent already being downloaded or "
                                     "seeded." ))
               except UserFailure, e:
                   # perform exception handling including shutting down
                   # the torrent.
                   self.got_exception(*sys.exc_info(),
                       **{'cannot_shutdown':True})
                   return
        self.reported_port = self.config['forwarded_port']
        if not self.reported_port:
            self.reported_port = \
                self._singleport_listener.get_port(self.change_port)
            if self.reported_port:
                self.reserved_ports.append(self.reported_port)

        # backward compatibility with older 5.0 development versions
        if self.destination_path == "":
            try:
                self.destination_path = self.config['save_as']
            except:
                pass
        if self.working_path == "":
            self.working_path = self.destination_path

        self._myid = self._make_id()
        random.seed(self._myid)
        self._build_url_mapping()
        self._urlage = URLage(self._urls)

        self._register_files()
        self.logger.debug("_start_download: self.working_path=%s", self.working_path)
        self._storage = Storage(self.config, self._filepool, self.working_path,
                                zip(self._myfiles, self.metainfo.sizes),
                                self.add_task, self.external_add_task,
                                self._doneflag)
        df = self._storage.startup_df
        yield df
        if df.getResult() != True:
            # initialization was aborted
            return

        resumefile = None
        if self.data_dir:
            filename = os.path.join(self.data_dir, 'resume',
                                    self.infohash.encode('hex'))
            if os.path.exists(filename):
                try:
                    resumefile = file(filename, 'rb')
                except Exception, e:
                    self._error(logging.WARNING,
                        _("Could not load fastresume data: %s") % unicode(e.args[0])
                        + ' ' + _("Will perform full hash check."))
                    if resumefile is not None:
                        resumefile.close()
                    resumefile = None
        def data_flunked(amount, index):
            self._ratemeasure.data_rejected(amount)
            self._error(logging.INFO,
                        _("piece %d failed hash check, re-downloading it")
                        % index)
        def errorfunc(level, text):
            def e():
                self._error(level, text)
            self.external_add_task(0, e)

        def statusfunc(activity = None, fractionDone = 0):
            if activity is None:
                activity = self._activity[0]
            self._activity = (activity, fractionDone)

        self._storagewrapper = StorageWrapper(self._storage, self.config,
                                              self.metainfo.hashes,
                                              self.metainfo.piece_length,
                                              statusfunc, self._doneflag,
                                              data_flunked, self.infohash,
                                              errorfunc, self.working_path,
                                              self.destination_path,
                                              resumefile,
                                              self.add_task,
                                              self.external_add_task)

        df = self._storagewrapper.done_checking_df
        yield df
        if df.getResult() != True:
            # initialization was aborted
            return

        if resumefile is not None:
            resumefile.close()

        numpieces = len(self.metainfo.hashes)

        self._upmeasure = Measure(self.config['max_rate_period'])
        self._downmeasure = Measure(self.config['max_rate_period'])
        self._ratemeasure = RateMeasure(self._storagewrapper.amount_left_with_partials)
        self._picker = PiecePicker(self.config, numpieces,
            self._storagewrapper.have_set.iterneg(0, numpieces))

        self._periodic_save_fastresume()

        while self._pending_file_priorities:
            self.set_file_priority(*self._pending_file_priorities.pop())

        def kickpeer(connection):
            def kick():
                connection.close()
            self.add_task(0, kick)
        def banpeer(ip):
            self._connection_manager.ban(ip)
        self.multidownload = MultiDownload(self.config, self._storagewrapper,
                                           self._urlage, self._picker,
                                           numpieces, self.finished,
                                           self._total_downmeasure,
                                           self._downmeasure,
                                           self._ratemeasure.data_came_in,
                                           kickpeer, banpeer)

        # HERE. Yipee! Uploads are created by callback while Download
        # objects are created by MultiDownload.   --Dave
        def make_upload(connection):
            return Upload(connection, self._ratelimiter, self._upmeasure,
                          self._choker, self._storagewrapper,
                          self.config['max_slice_length'],
                          self.config['max_rate_period'],
                          self.config['num_fast'], self)

        if self._dht:
            addContact = self._dht.addContact
        else:
            addContact = None

        df = self.metainfo.get_tracker_ips(_wrap_task(self.external_add_task))
        yield df
        tracker_ips = df.getResult()
        self._connection_manager = \
            ConnectionManager(make_upload, self.multidownload, self._choker,
                     numpieces, self._ratelimiter,
                     self._rawserver, self.config, self._myid,
                     self.add_task, self.infohash, self, addContact,
                     0, tracker_ips, self.log_root)
        self.multidownload.attach_connection_manager(self._connection_manager)

        self._statuscollector = TorrentStats(self.logger, self._choker,
            self.get_uprate, self.get_downrate, self._upmeasure.get_total,
            self._downmeasure.get_total, self._ratemeasure.get_time_left,
            self.get_percent_complete, self.multidownload.aggregate_piece_states,
            self.finflag, self._connection_manager, self.multidownload,
            self.get_file_priorities, self._myfiles,
            self._connection_manager.ever_got_incoming, None)

        for url_prefix in self.metainfo.url_list:
            r = urlparse.urlparse(url_prefix)
            host = r[1]
            if ':' in host:
                host, port = host.split(':')
                port = int(port)
            else:
                port = 80
            # TODO: async hostname resolution
            ip = socket.gethostbyname(host)
            self._connection_manager.start_http_connection((ip, port), url_prefix)

        self.state = "initialized"

    def start_download(self):
        assert self.state == "initialized"

        self.time_started = bttime()

        self._connection_manager.reopen(self.reported_port)

        self._singleport_listener.add_torrent(self.infohash,
                                              self._connection_manager)
        self._listening = True
        if self.metainfo.is_trackerless:
            if not self._dht:
                self._error(self, logging.CRITICAL,
                   _("Attempt to download a trackerless torrent "
                     "with trackerless client turned off."))
                return
            else:
                if len(self._dht.table.findNodes(self.metainfo.infohash,
                       invalid=False)) < const.K:
                    for ip, port in self.metainfo.nodes:
                        self._dht.addContact(ip, port)
                self._rerequest = DHTRerequester(self.config,
                    self.add_task,
                    self._connection_manager.how_many_connections,
                    self._connection_manager.start_connection,
                    self.external_add_task,
                    self._rawserver,
                    self._storagewrapper.get_amount_left,
                    self._upmeasure.get_total,
                    self._downmeasure.get_total, self.reported_port,
                    self._myid,
                    self.infohash, self._error, self.finflag,
                    self._upmeasure.get_rate,
                    self._downmeasure.get_rate,
                    self._connection_manager.ever_got_incoming,
                    self._no_announce_shutdown, self._announce_done,
                    self._dht)
        else:
            self._rerequest = Rerequester(self.metainfo.announce,
                self.metainfo.announce_list, self.config,
                self.add_task, self.external_add_task, self._rawserver,
                self._connection_manager.how_many_connections,
                self._connection_manager.start_connection,
                self._storagewrapper.get_amount_left,
                self._upmeasure.get_total, self._downmeasure.get_total,
                self.reported_port, self._myid,
                self.infohash, self._error, self.finflag,
                self._upmeasure.get_rate,
                self._downmeasure.get_rate,
                self._connection_manager.ever_got_incoming,
                self._no_announce_shutdown, self._announce_done)

        self._statuscollector.rerequester = self._rerequest
        self.multidownload.rerequester = self._rerequest

        self._announced = True
        if self._dht and len(self._dht.table.findNodes(self.infohash)) == 0:
            self.add_task(5, self._dht.findCloseNodes)
        self._rerequest.begin()

        self.state = "running"
        if not self.finflag.isSet():
            self._activity = (_("downloading"), 0)

        self.feedback.started(self)

        if self._storagewrapper.amount_left == 0 and not self.completed:
            # By default, self.finished() resets the policy to "auto",
            # but if we discover on startup that we are already finished,
            # we don't want to reset it.
            # Also, if we discover on startup that we are already finished,
            # don't set finished_this_session.
            self.finished(policy=self.policy, finished_this_session=False)

    def stop_download(self, pause=False):
        assert self.state == "running"

        self.state = "initialized"

        if not self.finflag.isSet():
            self._activity = (_("stopped"), 0)

        if self._announced:
            self._rerequest.announce_stop()
            self._announced = False
        self._statuscollector.rerequester = None
        self.multidownload.rerequester = None

        if self._listening:
            self._singleport_listener.remove_torrent(self.infohash)
            self._listening = False

        for port in self.reserved_ports:
            self._singleport_listener.release_port(port, self.change_port)
        del self.reserved_ports[:]

        if self._connection_manager is not None:
            if pause:
                self._connection_manager.throttle_connections()
            else:
                self._connection_manager.close_connections()

        if self.config['check_hashes']:
            self._save_fastresume()

    def shutdown(self):
        # use _rawserver.add_task directly here, because we want the callbacks
        # to happen even though _shutdown is about to invalidate this torrent's
        # context
        df = launch_coroutine(_wrap_task(self._rawserver.add_task), self._shutdown)
        df.addErrback(lambda e : self.got_exception(*e, **{'cannot_shutdown': True}))
        return df

    def _shutdown(self):
        self._doneflag.set()

        if self.state == "running":
            self.stop_download()
        # above is the last thing to set.

        if self._storagewrapper is not None:
            df = self._storagewrapper.done_checking_df
            yield df
            df.getResult()

        if self._storage is not None:
            df = self._storage.close()
            if df is not None:
                yield df
                df.getResult()

        self._unregister_files()

        self.context_valid = False

        self.state = "created"

        # release mutex on this torrent.
        if self.config["one_download_per_torrent"]:
           if self._mutex is not None and self._mutex.owner():
               self._mutex.release()

        self._rawserver.add_task(0, gc.collect)

    def _no_announce_shutdown(self, level, text):
        # This is only called when announce fails with no peers,
        # don't try to announce again telling we're leaving the torrent
        self._announced = False
        self._error(level, text)
        self.failed()

    def set_file_priority(self, filename, priority):
        if self._storagewrapper is None or self._picker is None:
            self._pending_file_priorities.append((filename, priority))
        else:
            begin, end = self._storagewrapper.get_piece_range_for_filename(filename)
            self._picker.set_priority(xrange(begin, end + 1), priority)
        self.config.setdefault('file_priorities', {})
        self.config['file_priorities'][filename] = priority
        self._dump_torrent_config()

    def get_file_priorities(self):
        return self.config.get('file_priorities', {})

    def get_file_priority(self, filename):
        fp = self.get_file_priorities()
        return fp.get(filename, 0)

    def got_exception(self, type, e, stack, cannot_shutdown=False):
        severity = logging.CRITICAL
        msg = "Torrent got exception: %s" % type
        try:
            e_str = unicode(e.args[0])
        except:
            e_str = str(e)
        if isinstance(e, BTFailure):
            self._activity = ( _("download failed: ") + e_str, 0)
        elif isinstance(e, IOError):
            if e.errno == errno.ENOSPC:
                msg = _("IO Error: No space left on disk, "
                        "or cannot create a file that large")
            self._activity = (_("killed by IO error: ") + e_str, 0)
        elif isinstance(e, OSError):
            self._activity = (_("killed by OS error: ") + e_str, 0)
        else:
            self._activity = (_("killed by internal exception: ") + e_str, 0)

        if isinstance(e, UserFailure):
            self.logger.log(severity, e_str )
        else:
            self.logger.log(severity, msg, exc_info=(type, e, stack))
        # steve wanted this too
        # Dave doesn't want it.
        #traceback.print_exception(type, e, stack, file=sys.stdout)

        self.failed(cannot_shutdown)

    def failed(self, cannot_shutdown=False):
        if cannot_shutdown:
            self.state = "failed"
        else:
            try:
                # this could complete later. sorry that's just the way it is.
                df = self.shutdown()
                def cb(*a):
                    self.state = "failed"
                df.addCallbacks(cb, cb)
            except:
                self.logger.exception(_("Additional error when closing down due to "
                                        "error: "))
        self.feedback.failed(self)

    def _error(self, level, text, exception=False, exc_info=None):
        if level > logging.WARNING:
            self.logger.log(level,
                            _('Error regarding "%s":\n')%self.metainfo.name + text,
                            exc_info=exc_info)
        if exception:
            self.feedback.exception(self, text)
        else:
            self.feedback.error(self, level, text)

    def finished(self, policy="auto", finished_this_session=True):
        assert self.state == "running"
        # because _finished() calls shutdown(), which invalidates the torrent
        # context, we need to use _rawserver.add_task directly here
        df = launch_coroutine(_wrap_task(self._rawserver.add_task), self._finished, policy=policy, finished_this_session=finished_this_session)
        df.addErrback(lambda e : self.got_exception(*e))
        return df

    def _finished(self, policy="auto", finished_this_session=True):
        if self.state != "running":
            return

        self.finflag.set()
        # Call self._storage.close() to flush buffers and change files to
        # read-only mode (when they're possibly reopened). Let exceptions
        # from self._storage.close() kill the torrent since files might not
        # be correct on disk if file.close() failed.
        self._storage.close()

        # don't bother trailing off the rate when we know we're done downloading
        self._downmeasure.rate = 0.0

        # If we haven't announced yet, normal first announce done later will
        # tell the tracker about seed status.
        if self._announced:
            self._rerequest.announce_finish()
        self._activity = (_("seeding"), 1)
        if self.config['check_hashes']:
            self._save_fastresume()

        # the old policy applied to downloading -- now that we are finished,
        # optionally reset it
        self.policy = policy

        self.feedback.finishing(self)

        config = self.config

        if finished_this_session:
            self.finished_this_session = True

        def move(working_path, destination_path):
            # this function is called from another thread, so don't do anything
            # that isn't thread safe in here

            self.logger.debug("deleting any file that might be in the way")
            try:
                os.remove(destination_path)
                self.logger.debug("successfully deleted file " +
                                  destination_path)
            except Exception, e:
                if os.path.exists(destination_path):
                    self.logger.debug(unicode(e.args[0]))

            self.logger.debug("deleting any directory that might be in the way")
            try:
                shutil.rmtree(destination_path)
                self.logger.debug("successfully deleted directory " +
                                  destination_path)
            except Exception, e:
                if os.path.exists(destination_path):
                    self.logger.debug(unicode(e.args[0]))

            self.logger.debug("actually moving file")
            shutil.move(working_path, destination_path)
            self.logger.debug("returned from move")


        if self.working_path != self.destination_path:
##            self.logger.debug("torrent finishing: shutting down, moving file, and restarting")
##            df = self.shutdown()
##            yield df
##            df.getResult()
            self.logger.debug("torrent finishing: pausing, moving file, and restarting")
            self.stop_download(pause=True)
            self._unregister_files()

            self.logger.debug("successfully paused torrent, moving file")

            self.state = "finishing"
            df = ThreadedDeferred(_wrap_task(self._rawserver.external_add_task),
                                  move, self.working_path, self.destination_path)
            yield df
            df.getResult()

            self.logger.debug("moved file, restarting")

            assert self.state == "finishing"
            self.working_path = self.destination_path
##            self.state = "created"
##            df = self.initialize()
##            yield df
##            df.getResult()
            self.completed = True
            self.feedback.finished(self)

            self.state = "initializing"
            self._register_files()
            df = self._storage.initialize(self.working_path,
                                          zip(self._myfiles,
                                              self.metainfo.sizes))
            yield df
            df.getResult()

            # so we store new path names
            self._storagewrapper.fastresume_dirty = True
            self._statuscollector.files = self._myfiles

            self.state = "initialized"
            self.logger.debug("attempting restart")
            self.start_download()
            self.logger.debug("re-started torrent")
        else:
            self.completed = True
            self.feedback.finished(self)

        self._dump_torrent_config()

    def fastresume_file_path(self):
        # HEREDAVE: should probably be self.data_dir?
        return os.path.join(self.config['data_dir'], 'resume',
                            self.infohash.encode('hex'))

    def config_file_path(self):
        return os.path.join(self.data_dir, 'torrents',
                            self.metainfo.infohash.encode('hex'))

    def _periodic_save_fastresume(self):
        self._save_fastresume()
        if not self.finflag.isSet():
            self.add_task(30, self._periodic_save_fastresume)

    def _save_fastresume(self):
        if not self.is_initialized():
            return
        # HEREDAVE: should probably be self.data_dir?
        if not self.config['data_dir']:
            return
        filename = self.fastresume_file_path()
        if os.path.exists(filename) and not self._storagewrapper.fastresume_dirty:
            return
        resumefile = None
        try:
            resumefile = file(filename, 'wb')
            self._storagewrapper.write_fastresume(resumefile)
            resumefile.close()
        except Exception, e:
            self._error(logging.WARNING, _("Could not write fastresume data: ") +
                        unicode(e.args[0]))
            if resumefile is not None:
                resumefile.close()

    def _dump_torrent_config(self):
        d = self.config.getDict()
##        nd = {}
##        for k,v in d.iteritems():
##            # can't bencode floats!
##            if not isinstance(v, float):
##                if isinstance(v, unicode):
##                    # FIXME -- what is the right thing to do here?
##                    v = v.encode('utf8')
##                nd[k] = v
##        s = bencode(nd)
        s = cPickle.dumps(d)
        path = self.config_file_path()
        f = file(path+'.new', 'wb')
        f.write(s)
        f.close()
        shutil.move(path+'.new', path)

    def remove_state_files(self, del_files=False):
        assert self.state == "created"

        try:
            os.remove(self.config_file_path())
        except Exception, e:
            self.logger.debug("error removing config file: %s", unicode(e.args[0]))

        try:
            os.remove(self.fastresume_file_path())
        except Exception, e:
            self.logger.debug("error removing fastresume file: %s", unicode(e.args[0]))

        if del_files:
            try:
                for file in self._last_myfiles:
                    try:
                        os.remove(file)
                    except OSError:
                        pass
                    d, f = os.path.split(file)
                    try:
                        os.rmdir(d)
                    except OSError:
                        pass

                try:
                    os.rmdir(self.working_path)
                except OSError:
                    pass
            except Exception, e:
                self.logger.debug("error removing incomplete files: %s", unicode(e.args[0]))

    def get_downrate(self):
        if self.is_running():
            return self._downmeasure.get_rate()

    def get_uprate(self):
        if self.is_running():
            return self._upmeasure.get_rate()

    def get_rates(self):
        return (self.get_uprate(), self.get_downrate())

    def get_downtotal(self):
        if self.is_running():
            return self._downmeasure.get_total()

    def get_uptotal(self):
        if self.is_running():
            return self._upmeasure.get_total()

    def get_percent_complete(self):
        if self.is_initialized():
            if self.total_bytes > 0:
                r = 1 - self._ratemeasure.get_size_left() / self.total_bytes
            else:
                r = 1.0
        else:
            r = 0.0

        return r

    def get_num_connections(self):
        if self._connection_manager:
            return self._connection_manager.how_many_connections()
        return 0

    def get_connections(self):
        return self._connection_manager.complete_connections

    def get_avg_peer_downrate(self):
        cs = self._connection_manager.complete_connections

        if len(cs) == 0:
            return 0.0

        total = 0.0
        for c in cs:
            total += c.download.connection.download.peermeasure.get_rate()

        return total / len(cs)

    def get_status(self, spew = False, fileinfo=False):
        if self.is_initialized():
            r = self._statuscollector.get_statistics(spew, fileinfo)
            r['activity'] = self._activity[0]
            r['priority'] = self.priority
            if not self.is_running():
                r['timeEst'] = None
        else:
            r = dict(itertools.izip(('activity', 'fractionDone'), self._activity))
            r['pieceStates'] = (0, 0, {})
            r['priority'] = self.priority
        return r

    def get_total_transfer(self):
        if self._upmeasure is None:
            return (0, 0)
        return (self._upmeasure.get_total(), self._downmeasure.get_total())

    def set_option(self, option, value):
        if self.config.has_key(option) and self.config[option] == value:
            return
        self.config[option] = value

    def change_port(self, new_port = None):
        r = self.config['forwarded_port']
        if r:
            for port in self.reserved_ports:
                self._singleport_listener.release_port(port)
            del self.reserved_ports[:]
            if self.rescrewedported_port == r:
                return
        elif new_port is not None:
            r = new_port
            self.reserved_ports.remove(self.reported_port)
            self.reserved_ports.append(r)
        elif self._singleport_listener.port != self.reported_port:
            r = self._singleport_listener.get_port(self.change_port)
            self.reserved_ports.append(r)
        else:
            return
        self.reported_port = r
        self._myid = self._make_id()
        if self._connection_manager:
            self._connection_manager.my_id = self._myid
        if self._announced:
            self._rerequest.change_port(self._myid, r)

    def _announce_done(self):
        for port in self.reserved_ports[:-1]:
            self._singleport_listener.release_port(port, self.change_port)
        del self.reserved_ports[:-1]

    def _make_id(self):
        return PeerID.make_id()



# The contents of this file are subject to the BitTorrent Open Source License
# Version 1.1 (the License).  You may not copy or use this file, in either
# source code or executable form, except in compliance with the License.  You
# may obtain a copy of the License at http://www.bittorrent.com/license/.
#
# Software distributed under the License is distributed on an AS IS basis,
# WITHOUT WARRANTY OF ANY KIND, either express or implied.  See the License
# for the specific language governing rights and limitations under the
# License.

#
#  knet.py
#  create a network of khashmir nodes
# usage: knet.py <num_nodes> <start_port> <ip_address>

from khashmir import Khashmir
from random import randrange
import sys, os

class Network:
    def __init__(self, size=0, startport=5555, localip='127.0.0.1'):
        self.num = size
        self.startport = startport
        self.localip = localip

    def _done(self, val):
        self.done = 1
        
    def setUp(self):
        self.kfiles()
        self.l = []
        for i in range(self.num):
            self.l.append(Khashmir('', self.startport + i, '/tmp/kh%s.db' % (self.startport + i)))
        reactor.iterate()
        reactor.iterate()
        
        for i in self.l:
            i.addContact(self.localip, self.l[randrange(0,self.num)].port)
            i.addContact(self.localip, self.l[randrange(0,self.num)].port)
            i.addContact(self.localip, self.l[randrange(0,self.num)].port)
            reactor.iterate()
            reactor.iterate()
            reactor.iterate() 
            
        for i in self.l:
            self.done = 0
            i.findCloseNodes(self._done)
            while not self.done:
                reactor.iterate()
        for i in self.l:
            self.done = 0
            i.findCloseNodes(self._done)
            while not self.done:
                reactor.iterate()

    def tearDown(self):
        for i in self.l:
            i.listenport.stopListening()
        self.kfiles()
        
    def kfiles(self):
        for i in range(self.startport, self.startport+self.num):
            try:
                os.unlink('/tmp/kh%s.db' % i)
            except:
                pass
            
        reactor.iterate()
    
if __name__ == "__main__":
    n = Network(int(sys.argv[1]), int(sys.argv[2]), sys.argv[3])
    n.setUp()
    try:
        reactor.run()
    finally:
        n.tearDown()
